#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2009 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc. 
#
import unittest
from tests.baseclass import *

from pykickstart.errors import *

class FC3_TestCase(CommandTest):
    command = "timezone"

    def runTest(self):
        # pass
        self.assert_parse("timezone Eastern", "timezone  Eastern\n")

        # On FC6 and later, we write out --isUtc regardless of what the input was.
        if self.__class__.__name__ == "FC3_TestCase":
            self.assert_parse("timezone --utc Eastern", "timezone --utc Eastern\n")
        else:
            self.assert_parse("timezone --utc Eastern", "timezone --isUtc Eastern\n")

        # fail
        self.assert_parse_error("timezone", KickstartValueError)
        self.assert_parse_error("timezone Eastern Central", KickstartValueError)
        self.assert_parse_error("timezone --blah Eastern")
        self.assert_parse_error("timezone --utc", KickstartValueError)

class FC6_TestCase(FC3_TestCase):
    def runTest(self):
        FC3_TestCase.runTest(self)

        # pass
        self.assert_parse("timezone --isUtc Eastern", "timezone --isUtc Eastern\n")

        # fail
        self.assert_parse_error("timezone --isUtc", KickstartValueError)

class F18_TestCase(FC6_TestCase):
    def runTest(self):
        # pass
        self.assert_parse("timezone --utc Europe/Prague")
        self.assert_parse("timezone --isUtc Europe/Prague\n")
        self.assert_parse("timezone --isUtc Eastern", "timezone Eastern --isUtc\n")
        self.assert_parse("timezone Europe/Prague")
        self.assert_parse("timezone Europe/Prague --nontp")
        self.assert_parse("timezone Europe/Prague "\
                          "--ntpservers=ntp.cesnet.cz,tik.nic.cz")
        self.assert_parse("timezone Europe/Prague --ntpservers=ntp.cesnet.cz,")

        # fail
        self.assert_parse_error("timezone", KickstartValueError)
        self.assert_parse_error("timezone Eastern Central", KickstartValueError)
        self.assert_parse_error("timezone --blah Eastern")
        self.assert_parse_error("timezone --utc", KickstartValueError)
        self.assert_parse_error("timezone --isUtc", KickstartValueError)
        self.assert_parse_error("timezone Europe/Prague --nontp "\
                                "--ntpservers=ntp.cesnet.cz",
                                KickstartParseError)
        self.assert_parse_error("timezone Europe/Prague --ntpservers="\
                                "ntp.cesnet.cz, tik.nic.cz",
                                KickstartValueError)

if __name__ == "__main__":
    unittest.main()



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2007, 2008, 2009, 2010, 2012, 2013 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc. 
#
from pykickstart.version import *
from pykickstart.commands import *

# This map is keyed on kickstart syntax version as provided by
# pykickstart.version.  Within each sub-dict is a mapping from command name
# to the class that handles it.  This is an onto mapping - that is, multiple
# command names can map to the same class.  However, the Handler will ensure
# that only one instance of each class ever exists.
commandMap = {
    FC3: {
        "auth": authconfig.FC3_Authconfig,
        "authconfig": authconfig.FC3_Authconfig,
        "autopart": autopart.FC3_AutoPart,
        "autostep": autostep.FC3_AutoStep,
        "bootloader": bootloader.FC3_Bootloader,
        "cdrom": cdrom.FC3_Cdrom,
        "clearpart": clearpart.FC3_ClearPart,
        "cmdline": displaymode.FC3_DisplayMode,
        "device": device.FC3_Device,
        "deviceprobe": deviceprobe.FC3_DeviceProbe,
        "driverdisk": driverdisk.FC3_DriverDisk,
        "firewall": firewall.FC3_Firewall,
        "firstboot": firstboot.FC3_Firstboot,
        "graphical": displaymode.FC3_DisplayMode,
        "halt": reboot.FC3_Reboot,
        "harddrive": harddrive.FC3_HardDrive,
        "ignoredisk": ignoredisk.FC3_IgnoreDisk,
        "install": upgrade.FC3_Upgrade,
        "interactive": interactive.FC3_Interactive,
        "keyboard": keyboard.FC3_Keyboard,
        "lang": lang.FC3_Lang,
        "langsupport": langsupport.FC3_LangSupport,
        "lilo": bootloader.FC3_Bootloader,
        "lilocheck": lilocheck.FC3_LiloCheck,
        "logvol": logvol.FC3_LogVol,
        "method": method.FC3_Method,
        "monitor": monitor.FC3_Monitor,
        "mouse": mouse.FC3_Mouse,
        "network": network.FC3_Network,
        "nfs": nfs.FC3_NFS,
        "part": partition.FC3_Partition,
        "partition": partition.FC3_Partition,
        "poweroff": reboot.FC3_Reboot,
        "raid": raid.FC3_Raid,
        "reboot": reboot.FC3_Reboot,
        "rootpw": rootpw.FC3_RootPw,
        "selinux": selinux.FC3_SELinux,
        "shutdown": reboot.FC3_Reboot,
        "skipx": skipx.FC3_SkipX,
        "text": displaymode.FC3_DisplayMode,
        "timezone": timezone.FC3_Timezone,
        "upgrade": upgrade.FC3_Upgrade,
        "url": url.FC3_Url,
        "vnc": vnc.FC3_Vnc,
        "volgroup": volgroup.FC3_VolGroup,
        "xconfig": xconfig.FC3_XConfig,
        "zerombr": zerombr.FC3_ZeroMbr,
        "zfcp": zfcp.FC3_ZFCP,
    },

    # based on fc3
    FC4: {
        "auth": authconfig.FC3_Authconfig,
        "authconfig": authconfig.FC3_Authconfig,
        "autopart": autopart.FC3_AutoPart,
        "autostep": autostep.FC3_AutoStep,
        "bootloader": bootloader.FC4_Bootloader,
        "cdrom": cdrom.FC3_Cdrom,
        "clearpart": clearpart.FC3_ClearPart,
        "cmdline": displaymode.FC3_DisplayMode,
        "device": device.FC3_Device,
        "deviceprobe": deviceprobe.FC3_DeviceProbe,
        "driverdisk": driverdisk.FC4_DriverDisk,
        "firewall": firewall.FC3_Firewall,
        "firstboot": firstboot.FC3_Firstboot,
        "graphical": displaymode.FC3_DisplayMode,
        "halt": reboot.FC3_Reboot,
        "harddrive": harddrive.FC3_HardDrive,
        "ignoredisk": ignoredisk.FC3_IgnoreDisk,
        "install": upgrade.FC3_Upgrade,
        "interactive": interactive.FC3_Interactive,
        "keyboard": keyboard.FC3_Keyboard,
        "lang": lang.FC3_Lang,
        "langsupport": langsupport.FC3_LangSupport,
        "logvol": logvol.FC4_LogVol,
        "mediacheck": mediacheck.FC4_MediaCheck,
        "method": method.FC3_Method,
        "monitor": monitor.FC3_Monitor,
        "mouse": mouse.FC3_Mouse,
        "network": network.FC4_Network,
        "nfs": nfs.FC3_NFS,
        "part": partition.FC4_Partition,
        "partition": partition.FC4_Partition,
        "poweroff": reboot.FC3_Reboot,
        "raid": raid.FC4_Raid,
        "reboot": reboot.FC3_Reboot,
        "rootpw": rootpw.FC3_RootPw,
        "selinux": selinux.FC3_SELinux,
        "shutdown": reboot.FC3_Reboot,
        "skipx": skipx.FC3_SkipX,
        "text": displaymode.FC3_DisplayMode,
        "timezone": timezone.FC3_Timezone,
        "upgrade": upgrade.FC3_Upgrade,
        "url": url.FC3_Url,
        "vnc": vnc.FC3_Vnc,
        "volgroup": volgroup.FC3_VolGroup,
        "xconfig": xconfig.FC3_XConfig,
        "zerombr": zerombr.FC3_ZeroMbr,
        "zfcp": zfcp.FC3_ZFCP,
    },

    # based on fc4
    FC5: {
        "auth": authconfig.FC3_Authconfig,
        "authconfig": authconfig.FC3_Authconfig,
        "autopart": autopart.FC3_AutoPart,
        "autostep": autostep.FC3_AutoStep,
        "bootloader": bootloader.FC4_Bootloader,
        "cdrom": cdrom.FC3_Cdrom,
        "clearpart": clearpart.FC3_ClearPart,
        "cmdline": displaymode.FC3_DisplayMode,
        "device": device.FC3_Device,
        "deviceprobe": deviceprobe.FC3_DeviceProbe,
        "driverdisk": driverdisk.FC4_DriverDisk,
        "firewall": firewall.FC3_Firewall,
        "firstboot": firstboot.FC3_Firstboot,
        "graphical": displaymode.FC3_DisplayMode,
        "halt": reboot.FC3_Reboot,
        "harddrive": harddrive.FC3_HardDrive,
        "ignoredisk": ignoredisk.FC3_IgnoreDisk,
        "install": upgrade.FC3_Upgrade,
        "interactive": interactive.FC3_Interactive,
        "keyboard": keyboard.FC3_Keyboard,
        "lang": lang.FC3_Lang,
        "langsupport": langsupport.FC5_LangSupport,
        "logvol": logvol.FC4_LogVol,
        "mediacheck": mediacheck.FC4_MediaCheck,
        "method": method.FC3_Method,
        "monitor": monitor.FC3_Monitor,
        "mouse": mouse.FC3_Mouse,
        "network": network.FC4_Network,
        "nfs": nfs.FC3_NFS,
        "part": partition.FC4_Partition,
        "partition": partition.FC4_Partition,
        "poweroff": reboot.FC3_Reboot,
        "raid": raid.FC5_Raid,
        "reboot": reboot.FC3_Reboot,
        "rootpw": rootpw.FC3_RootPw,
        "selinux": selinux.FC3_SELinux,
        "shutdown": reboot.FC3_Reboot,
        "skipx": skipx.FC3_SkipX,
        "text": displaymode.FC3_DisplayMode,
        "timezone": timezone.FC3_Timezone,
        "upgrade": upgrade.FC3_Upgrade,
        "url": url.FC3_Url,
        "vnc": vnc.FC3_Vnc,
        "volgroup": volgroup.FC3_VolGroup,
        "xconfig": xconfig.FC3_XConfig,
        "zerombr": zerombr.FC3_ZeroMbr,
        "zfcp": zfcp.FC3_ZFCP,
    },

    # based on fc5
    FC6: {
        "auth": authconfig.FC3_Authconfig,
        "authconfig": authconfig.FC3_Authconfig,
        "autopart": autopart.FC3_AutoPart,
        "autostep": autostep.FC3_AutoStep,
        "bootloader": bootloader.FC4_Bootloader,
        "cdrom": cdrom.FC3_Cdrom,
        "clearpart": clearpart.FC3_ClearPart,
        "cmdline": displaymode.FC3_DisplayMode,
        "device": device.FC3_Device,
        "deviceprobe": deviceprobe.FC3_DeviceProbe,
        "dmraid": dmraid.FC6_DmRaid,
        "driverdisk": driverdisk.FC4_DriverDisk,
        "firewall": firewall.FC3_Firewall,
        "firstboot": firstboot.FC3_Firstboot,
        "graphical": displaymode.FC3_DisplayMode,
        "halt": reboot.FC6_Reboot,
        "harddrive": harddrive.FC3_HardDrive,
        "ignoredisk": ignoredisk.FC3_IgnoreDisk,
        "install": upgrade.FC3_Upgrade,
        "interactive": interactive.FC3_Interactive,
        "iscsi": iscsi.FC6_Iscsi,
        "iscsiname": iscsiname.FC6_IscsiName,
        "keyboard": keyboard.FC3_Keyboard,
        "lang": lang.FC3_Lang,
        "langsupport": langsupport.FC5_LangSupport,
        "logging": logging.FC6_Logging,
        "logvol": logvol.FC4_LogVol,
        "mediacheck": mediacheck.FC4_MediaCheck,
        "method": method.FC6_Method,
        "monitor": monitor.FC6_Monitor,
        "mouse": mouse.FC3_Mouse,
        "multipath": multipath.FC6_MultiPath,
        "network": network.FC6_Network,
        "nfs": nfs.FC6_NFS,
        "part": partition.FC4_Partition,
        "partition": partition.FC4_Partition,
        "poweroff": reboot.FC6_Reboot,
        "raid": raid.FC5_Raid,
        "reboot": reboot.FC6_Reboot,
        "repo": repo.FC6_Repo,
        "rootpw": rootpw.FC3_RootPw,
        "selinux": selinux.FC3_SELinux,
        "services": services.FC6_Services,
        "shutdown": reboot.FC6_Reboot,
        "skipx": skipx.FC3_SkipX,
        "text": displaymode.FC3_DisplayMode,
        "timezone": timezone.FC6_Timezone,
        "upgrade": upgrade.FC3_Upgrade,
        "user": user.FC6_User,
        "url": url.FC3_Url,
        "vnc": vnc.FC6_Vnc,
        "volgroup": volgroup.FC3_VolGroup,
        "xconfig": xconfig.FC6_XConfig,
        "zerombr": zerombr.FC3_ZeroMbr,
        "zfcp": zfcp.FC3_ZFCP,
    },

    # based on fc6
    F7: {
        "auth": authconfig.FC3_Authconfig,
        "authconfig": authconfig.FC3_Authconfig,
        "autopart": autopart.FC3_AutoPart,
        "autostep": autostep.FC3_AutoStep,
        "bootloader": bootloader.FC4_Bootloader,
        "cdrom": cdrom.FC3_Cdrom,
        "clearpart": clearpart.FC3_ClearPart,
        "cmdline": displaymode.FC3_DisplayMode,
        "device": device.FC3_Device,
        "deviceprobe": deviceprobe.FC3_DeviceProbe,
        "dmraid": dmraid.FC6_DmRaid,
        "driverdisk": driverdisk.FC4_DriverDisk,
        "firewall": firewall.FC3_Firewall,
        "firstboot": firstboot.FC3_Firstboot,
        "graphical": displaymode.FC3_DisplayMode,
        "halt": reboot.FC6_Reboot,
        "harddrive": harddrive.FC3_HardDrive,
        "ignoredisk": ignoredisk.FC3_IgnoreDisk,
        "install": upgrade.FC3_Upgrade,
        "interactive": interactive.FC3_Interactive,
        "iscsi": iscsi.FC6_Iscsi,
        "iscsiname": iscsiname.FC6_IscsiName,
        "keyboard": keyboard.FC3_Keyboard,
        "lang": lang.FC3_Lang,
        "logging": logging.FC6_Logging,
        "logvol": logvol.FC4_LogVol,
        "mediacheck": mediacheck.FC4_MediaCheck,
        "method": method.FC6_Method,
        "monitor": monitor.FC6_Monitor,
        "multipath": multipath.FC6_MultiPath,
        "network": network.FC6_Network,
        "nfs": nfs.FC6_NFS,
        "part": partition.FC4_Partition,
        "partition": partition.FC4_Partition,
        "poweroff": reboot.FC6_Reboot,
        "raid": raid.F7_Raid,
        "reboot": reboot.FC6_Reboot,
        "repo": repo.FC6_Repo,
        "rootpw": rootpw.FC3_RootPw,
        "selinux": selinux.FC3_SELinux,
        "services": services.FC6_Services,
        "shutdown": reboot.FC6_Reboot,
        "skipx": skipx.FC3_SkipX,
        "text": displaymode.FC3_DisplayMode,
        "timezone": timezone.FC6_Timezone,
        "updates": updates.F7_Updates,
        "upgrade": upgrade.FC3_Upgrade,
        "url": url.FC3_Url,
        "user": user.FC6_User,
        "vnc": vnc.FC6_Vnc,
        "volgroup": volgroup.FC3_VolGroup,
        "xconfig": xconfig.FC6_XConfig,
        "zerombr": zerombr.FC3_ZeroMbr,
        "zfcp": zfcp.FC3_ZFCP,
    },

    # based on f7
    F8: {
        "auth": authconfig.FC3_Authconfig,
        "authconfig": authconfig.FC3_Authconfig,
        "autopart": autopart.FC3_AutoPart,
        "autostep": autostep.FC3_AutoStep,
        "bootloader": bootloader.F8_Bootloader,
        "cdrom": cdrom.FC3_Cdrom,
        "clearpart": clearpart.FC3_ClearPart,
        "cmdline": displaymode.FC3_DisplayMode,
        "device": device.F8_Device,
        "deviceprobe": deviceprobe.FC3_DeviceProbe,
        "dmraid": dmraid.FC6_DmRaid,
        "driverdisk": driverdisk.FC4_DriverDisk,
        "firewall": firewall.FC3_Firewall,
        "firstboot": firstboot.FC3_Firstboot,
        "graphical": displaymode.FC3_DisplayMode,
        "halt": reboot.FC6_Reboot,
        "harddrive": harddrive.FC3_HardDrive,
        "ignoredisk": ignoredisk.F8_IgnoreDisk,
        "install": upgrade.FC3_Upgrade,
        "interactive": interactive.FC3_Interactive,
        "iscsi": iscsi.FC6_Iscsi,
        "iscsiname": iscsiname.FC6_IscsiName,
        "keyboard": keyboard.FC3_Keyboard,
        "lang": lang.FC3_Lang,
        "logging": logging.FC6_Logging,
        "logvol": logvol.FC4_LogVol,
        "mediacheck": mediacheck.FC4_MediaCheck,
        "method": method.FC6_Method,
        "monitor": monitor.FC6_Monitor,
        "multipath": multipath.FC6_MultiPath,
        "network": network.F8_Network,
        "nfs": nfs.FC6_NFS,
        "part": partition.FC4_Partition,
        "partition": partition.FC4_Partition,
        "poweroff": reboot.FC6_Reboot,
        "raid": raid.F7_Raid,
        "reboot": reboot.FC6_Reboot,
        "repo": repo.F8_Repo,
        "rootpw": rootpw.F8_RootPw,
        "selinux": selinux.FC3_SELinux,
        "services": services.FC6_Services,
        "shutdown": reboot.FC6_Reboot,
        "skipx": skipx.FC3_SkipX,
        "text": displaymode.FC3_DisplayMode,
        "timezone": timezone.FC6_Timezone,
        "updates": updates.F7_Updates,
        "upgrade": upgrade.FC3_Upgrade,
        "url": url.FC3_Url,
        "user": user.F8_User,
        "vnc": vnc.FC6_Vnc,
        "volgroup": volgroup.FC3_VolGroup,
        "xconfig": xconfig.FC6_XConfig,
        "zerombr": zerombr.FC3_ZeroMbr,
        "zfcp": zfcp.FC3_ZFCP,
    },

    # based on f8
    F9: {
        "auth": authconfig.FC3_Authconfig,
        "authconfig": authconfig.FC3_Authconfig,
        "autopart": autopart.F9_AutoPart,
        "autostep": autostep.FC3_AutoStep,
        "bootloader": bootloader.F8_Bootloader,
        "cdrom": cdrom.FC3_Cdrom,
        "clearpart": clearpart.FC3_ClearPart,
        "cmdline": displaymode.FC3_DisplayMode,
        "device": device.F8_Device,
        "deviceprobe": deviceprobe.FC3_DeviceProbe,
        "dmraid": dmraid.FC6_DmRaid,
        "driverdisk": driverdisk.FC4_DriverDisk,
        "firewall": firewall.F9_Firewall,
        "firstboot": firstboot.FC3_Firstboot,
        "graphical": displaymode.FC3_DisplayMode,
        "halt": reboot.FC6_Reboot,
        "harddrive": harddrive.FC3_HardDrive,
        "ignoredisk": ignoredisk.F8_IgnoreDisk,
        "install": upgrade.FC3_Upgrade,
        "interactive": interactive.FC3_Interactive,
        "iscsi": iscsi.FC6_Iscsi,
        "iscsiname": iscsiname.FC6_IscsiName,
        "keyboard": keyboard.FC3_Keyboard,
        "lang": lang.FC3_Lang,
        "logging": logging.FC6_Logging,
        "logvol": logvol.F9_LogVol,
        "mediacheck": mediacheck.FC4_MediaCheck,
        "method": method.FC6_Method,
        "monitor": monitor.FC6_Monitor,
        "multipath": multipath.FC6_MultiPath,
        "network": network.F9_Network,
        "nfs": nfs.FC6_NFS,
        "part": partition.F9_Partition,
        "partition": partition.F9_Partition,
        "poweroff": reboot.FC6_Reboot,
        "raid": raid.F9_Raid,
        "reboot": reboot.FC6_Reboot,
        "repo": repo.F8_Repo,
        "rootpw": rootpw.F8_RootPw,
        "selinux": selinux.FC3_SELinux,
        "services": services.FC6_Services,
        "shutdown": reboot.FC6_Reboot,
        "skipx": skipx.FC3_SkipX,
        "text": displaymode.FC3_DisplayMode,
        "timezone": timezone.FC6_Timezone,
        "updates": updates.F7_Updates,
        "upgrade": upgrade.FC3_Upgrade,
        "url": url.FC3_Url,
        "user": user.F8_User,
        "vnc": vnc.F9_Vnc,
        "volgroup": volgroup.FC3_VolGroup,
        "xconfig": xconfig.F9_XConfig,
        "zerombr": zerombr.F9_ZeroMbr,
        "zfcp": zfcp.FC3_ZFCP,
    },

    # based on f9
    F10: {
        "auth": authconfig.FC3_Authconfig,
        "authconfig": authconfig.FC3_Authconfig,
        "autopart": autopart.F9_AutoPart,
        "autostep": autostep.FC3_AutoStep,
        "bootloader": bootloader.F8_Bootloader,
        "cdrom": cdrom.FC3_Cdrom,
        "clearpart": clearpart.FC3_ClearPart,
        "cmdline": displaymode.FC3_DisplayMode,
        "device": device.F8_Device,
        "deviceprobe": deviceprobe.FC3_DeviceProbe,
        "dmraid": dmraid.FC6_DmRaid,
        "driverdisk": driverdisk.FC4_DriverDisk,
        "firewall": firewall.F10_Firewall,
        "firstboot": firstboot.FC3_Firstboot,
        "graphical": displaymode.FC3_DisplayMode,
        "halt": reboot.FC6_Reboot,
        "harddrive": harddrive.FC3_HardDrive,
        "ignoredisk": ignoredisk.F8_IgnoreDisk,
        "install": upgrade.FC3_Upgrade,
        "interactive": interactive.FC3_Interactive,
        "iscsi": iscsi.F10_Iscsi,
        "iscsiname": iscsiname.FC6_IscsiName,
        "keyboard": keyboard.FC3_Keyboard,
        "lang": lang.FC3_Lang,
        "logging": logging.FC6_Logging,
        "logvol": logvol.F9_LogVol,
        "mediacheck": mediacheck.FC4_MediaCheck,
        "method": method.FC6_Method,
        "monitor": monitor.F10_Monitor,
        "multipath": multipath.FC6_MultiPath,
        "network": network.F9_Network,
        "nfs": nfs.FC6_NFS,
        "part": partition.F9_Partition,
        "partition": partition.F9_Partition,
        "poweroff": reboot.FC6_Reboot,
        "raid": raid.F9_Raid,
        "reboot": reboot.FC6_Reboot,
        "repo": repo.F8_Repo,
        "rescue": rescue.F10_Rescue,
        "rootpw": rootpw.F8_RootPw,
        "selinux": selinux.FC3_SELinux,
        "services": services.FC6_Services,
        "shutdown": reboot.FC6_Reboot,
        "skipx": skipx.FC3_SkipX,
        "text": displaymode.FC3_DisplayMode,
        "timezone": timezone.FC6_Timezone,
        "updates": updates.F7_Updates,
        "upgrade": upgrade.FC3_Upgrade,
        "url": url.FC3_Url,
        "user": user.F8_User,
        "vnc": vnc.F9_Vnc,
        "volgroup": volgroup.FC3_VolGroup,
        "xconfig": xconfig.F10_XConfig,
        "zerombr": zerombr.F9_ZeroMbr,
        "zfcp": zfcp.FC3_ZFCP,
    },

    # based on f10
    F11: {
        "auth": authconfig.FC3_Authconfig,
        "authconfig": authconfig.FC3_Authconfig,
        "autopart": autopart.F9_AutoPart,
        "autostep": autostep.FC3_AutoStep,
        "bootloader": bootloader.F8_Bootloader,
        "cdrom": cdrom.FC3_Cdrom,
        "clearpart": clearpart.FC3_ClearPart,
        "cmdline": displaymode.FC3_DisplayMode,
        "device": device.F8_Device,
        "deviceprobe": deviceprobe.FC3_DeviceProbe,
        "dmraid": dmraid.FC6_DmRaid,
        "driverdisk": driverdisk.FC4_DriverDisk,
        "firewall": firewall.F10_Firewall,
        "firstboot": firstboot.FC3_Firstboot,
        "graphical": displaymode.FC3_DisplayMode,
        "halt": reboot.FC6_Reboot,
        "harddrive": harddrive.FC3_HardDrive,
        "ignoredisk": ignoredisk.F8_IgnoreDisk,
        "install": upgrade.F11_Upgrade,
        "interactive": interactive.FC3_Interactive,
        "iscsi": iscsi.F10_Iscsi,
        "iscsiname": iscsiname.FC6_IscsiName,
        "keyboard": keyboard.FC3_Keyboard,
        "lang": lang.FC3_Lang,
        "logging": logging.FC6_Logging,
        "logvol": logvol.F9_LogVol,
        "mediacheck": mediacheck.FC4_MediaCheck,
        "method": method.FC6_Method,
        "monitor": monitor.F10_Monitor,
        "multipath": multipath.FC6_MultiPath,
        "network": network.F9_Network,
        "nfs": nfs.FC6_NFS,
        "part": partition.F11_Partition,
        "partition": partition.F11_Partition,
        "poweroff": reboot.FC6_Reboot,
        "raid": raid.F9_Raid,
        "reboot": reboot.FC6_Reboot,
        "repo": repo.F11_Repo,
        "rescue": rescue.F10_Rescue,
        "rootpw": rootpw.F8_RootPw,
        "selinux": selinux.FC3_SELinux,
        "services": services.FC6_Services,
        "shutdown": reboot.FC6_Reboot,
        "skipx": skipx.FC3_SkipX,
        "text": displaymode.FC3_DisplayMode,
        "timezone": timezone.FC6_Timezone,
        "updates": updates.F7_Updates,
        "upgrade": upgrade.F11_Upgrade,
        "url": url.FC3_Url,
        "user": user.F8_User,
        "vnc": vnc.F9_Vnc,
        "volgroup": volgroup.FC3_VolGroup,
        "xconfig": xconfig.F10_XConfig,
        "zerombr": zerombr.F9_ZeroMbr,
        "zfcp": zfcp.FC3_ZFCP,
    },

    # based on f11
    F12: {
        "auth": authconfig.FC3_Authconfig,
        "authconfig": authconfig.FC3_Authconfig,
        "autopart": autopart.F12_AutoPart,
        "autostep": autostep.FC3_AutoStep,
        "bootloader": bootloader.F12_Bootloader,
        "cdrom": cdrom.FC3_Cdrom,
        "clearpart": clearpart.FC3_ClearPart,
        "cmdline": displaymode.FC3_DisplayMode,
        "device": device.F8_Device,
        "deviceprobe": deviceprobe.FC3_DeviceProbe,
        "dmraid": dmraid.FC6_DmRaid,
        "driverdisk": driverdisk.F12_DriverDisk,
        "fcoe": fcoe.F12_Fcoe,
        "firewall": firewall.F10_Firewall,
        "firstboot": firstboot.FC3_Firstboot,
        "graphical": displaymode.FC3_DisplayMode,
        "group": group.F12_Group,
        "halt": reboot.FC6_Reboot,
        "harddrive": harddrive.FC3_HardDrive,
        "ignoredisk": ignoredisk.F8_IgnoreDisk,
        "install": upgrade.F11_Upgrade,
        "interactive": interactive.FC3_Interactive,
        "iscsi": iscsi.F10_Iscsi,
        "iscsiname": iscsiname.FC6_IscsiName,
        "keyboard": keyboard.FC3_Keyboard,
        "lang": lang.FC3_Lang,
        "logging": logging.FC6_Logging,
        "logvol": logvol.F12_LogVol,
        "mediacheck": mediacheck.FC4_MediaCheck,
        "method": method.FC6_Method,
        "monitor": monitor.F10_Monitor,
        "multipath": multipath.FC6_MultiPath,
        "network": network.F9_Network,
        "nfs": nfs.FC6_NFS,
        "part": partition.F12_Partition,
        "partition": partition.F12_Partition,
        "poweroff": reboot.FC6_Reboot,
        "raid": raid.F12_Raid,
        "reboot": reboot.FC6_Reboot,
        "repo": repo.F11_Repo,
        "rescue": rescue.F10_Rescue,
        "rootpw": rootpw.F8_RootPw,
        "selinux": selinux.FC3_SELinux,
        "services": services.FC6_Services,
        "shutdown": reboot.FC6_Reboot,
        "skipx": skipx.FC3_SkipX,
        "text": displaymode.FC3_DisplayMode,
        "timezone": timezone.FC6_Timezone,
        "updates": updates.F7_Updates,
        "upgrade": upgrade.F11_Upgrade,
        "url": url.FC3_Url,
        "user": user.F12_User,
        "vnc": vnc.F9_Vnc,
        "volgroup": volgroup.FC3_VolGroup,
        "xconfig": xconfig.F10_XConfig,
        "zerombr": zerombr.F9_ZeroMbr,
        "zfcp": zfcp.F12_ZFCP,
    },

    # based on f12
    F13: {
        "auth": authconfig.FC3_Authconfig,
        "authconfig": authconfig.FC3_Authconfig,
        "autopart": autopart.F12_AutoPart,
        "autostep": autostep.FC3_AutoStep,
        "bootloader": bootloader.F12_Bootloader,
        "cdrom": cdrom.FC3_Cdrom,
        "clearpart": clearpart.FC3_ClearPart,
        "cmdline": displaymode.FC3_DisplayMode,
        "device": device.F8_Device,
        "deviceprobe": deviceprobe.FC3_DeviceProbe,
        "dmraid": dmraid.FC6_DmRaid,
        "driverdisk": driverdisk.F12_DriverDisk,
        "fcoe": fcoe.F13_Fcoe,
        "firewall": firewall.F10_Firewall,
        "firstboot": firstboot.FC3_Firstboot,
        "graphical": displaymode.FC3_DisplayMode,
        "group": group.F12_Group,
        "halt": reboot.FC6_Reboot,
        "harddrive": harddrive.FC3_HardDrive,
        "ignoredisk": ignoredisk.F8_IgnoreDisk,
        "install": upgrade.F11_Upgrade,
        "interactive": interactive.FC3_Interactive,
        "iscsi": iscsi.F10_Iscsi,
        "iscsiname": iscsiname.FC6_IscsiName,
        "keyboard": keyboard.FC3_Keyboard,
        "lang": lang.FC3_Lang,
        "logging": logging.FC6_Logging,
        "logvol": logvol.F12_LogVol,
        "mediacheck": mediacheck.FC4_MediaCheck,
        "method": method.F13_Method,
        "monitor": monitor.F10_Monitor,
        "multipath": multipath.FC6_MultiPath,
        "network": network.F9_Network,
        "nfs": nfs.FC6_NFS,
        "part": partition.F12_Partition,
        "partition": partition.F12_Partition,
        "poweroff": reboot.FC6_Reboot,
        "raid": raid.F13_Raid,
        "reboot": reboot.FC6_Reboot,
        "repo": repo.F13_Repo,
        "rescue": rescue.F10_Rescue,
        "rootpw": rootpw.F8_RootPw,
        "selinux": selinux.FC3_SELinux,
        "services": services.FC6_Services,
        "shutdown": reboot.FC6_Reboot,
        "skipx": skipx.FC3_SkipX,
        "sshpw": sshpw.F13_SshPw,
        "text": displaymode.FC3_DisplayMode,
        "timezone": timezone.FC6_Timezone,
        "updates": updates.F7_Updates,
        "upgrade": upgrade.F11_Upgrade,
        "url": url.F13_Url,
        "user": user.F12_User,
        "vnc": vnc.F9_Vnc,
        "volgroup": volgroup.FC3_VolGroup,
        "xconfig": xconfig.F10_XConfig,
        "zerombr": zerombr.F9_ZeroMbr,
        "zfcp": zfcp.F12_ZFCP,
    },

    # based on f13
    F14: {
        "auth": authconfig.FC3_Authconfig,
        "authconfig": authconfig.FC3_Authconfig,
        "autopart": autopart.F12_AutoPart,
        "autostep": autostep.FC3_AutoStep,
        "bootloader": bootloader.F14_Bootloader,
        "cdrom": cdrom.FC3_Cdrom,
        "clearpart": clearpart.FC3_ClearPart,
        "cmdline": displaymode.FC3_DisplayMode,
        "device": device.F8_Device,
        "deviceprobe": deviceprobe.FC3_DeviceProbe,
        "dmraid": dmraid.FC6_DmRaid,
        "driverdisk": driverdisk.F14_DriverDisk,
        "fcoe": fcoe.F13_Fcoe,
        "firewall": firewall.F14_Firewall,
        "firstboot": firstboot.FC3_Firstboot,
        "graphical": displaymode.FC3_DisplayMode,
        "group": group.F12_Group,
        "halt": reboot.FC6_Reboot,
        "harddrive": harddrive.FC3_HardDrive,
        "ignoredisk": ignoredisk.F14_IgnoreDisk,
        "install": upgrade.F11_Upgrade,
        "interactive": interactive.F14_Interactive,
        "iscsi": iscsi.F10_Iscsi,
        "iscsiname": iscsiname.FC6_IscsiName,
        "keyboard": keyboard.FC3_Keyboard,
        "lang": lang.FC3_Lang,
        "logging": logging.FC6_Logging,
        "logvol": logvol.F14_LogVol,
        "mediacheck": mediacheck.FC4_MediaCheck,
        "method": method.F14_Method,
        "monitor": monitor.F10_Monitor,
        "multipath": multipath.FC6_MultiPath,
        "network": network.F9_Network,
        "nfs": nfs.FC6_NFS,
        "part": partition.F14_Partition,
        "partition": partition.F14_Partition,
        "poweroff": reboot.FC6_Reboot,
        "raid": raid.F14_Raid,
        "reboot": reboot.FC6_Reboot,
        "repo": repo.F14_Repo,
        "rescue": rescue.F10_Rescue,
        "rootpw": rootpw.F8_RootPw,
        "selinux": selinux.FC3_SELinux,
        "services": services.FC6_Services,
        "shutdown": reboot.FC6_Reboot,
        "skipx": skipx.FC3_SkipX,
        "sshpw": sshpw.F13_SshPw,
        "text": displaymode.FC3_DisplayMode,
        "timezone": timezone.FC6_Timezone,
        "updates": updates.F7_Updates,
        "upgrade": upgrade.F11_Upgrade,
        "url": url.F14_Url,
        "user": user.F12_User,
        "vnc": vnc.F9_Vnc,
        "volgroup": volgroup.FC3_VolGroup,
        "xconfig": xconfig.F14_XConfig,
        "zerombr": zerombr.F9_ZeroMbr,
        "zfcp": zfcp.F14_ZFCP,
    },

    # based on f14
    F15: {
        "auth": authconfig.FC3_Authconfig,
        "authconfig": authconfig.FC3_Authconfig,
        "autopart": autopart.F12_AutoPart,
        "autostep": autostep.FC3_AutoStep,
        "bootloader": bootloader.F15_Bootloader,
        "cdrom": cdrom.FC3_Cdrom,
        "clearpart": clearpart.FC3_ClearPart,
        "cmdline": displaymode.FC3_DisplayMode,
        "device": device.F8_Device,
        "deviceprobe": deviceprobe.FC3_DeviceProbe,
        "dmraid": dmraid.FC6_DmRaid,
        "driverdisk": driverdisk.F14_DriverDisk,
        "fcoe": fcoe.F13_Fcoe,
        "firewall": firewall.F14_Firewall,
        "firstboot": firstboot.FC3_Firstboot,
        "graphical": displaymode.FC3_DisplayMode,
        "group": group.F12_Group,
        "halt": reboot.FC6_Reboot,
        "harddrive": harddrive.FC3_HardDrive,
        "ignoredisk": ignoredisk.F14_IgnoreDisk,
        "install": upgrade.F11_Upgrade,
        "iscsi": iscsi.F10_Iscsi,
        "iscsiname": iscsiname.FC6_IscsiName,
        "keyboard": keyboard.FC3_Keyboard,
        "lang": lang.FC3_Lang,
        "logging": logging.FC6_Logging,
        "logvol": logvol.F15_LogVol,
        "mediacheck": mediacheck.FC4_MediaCheck,
        "method": method.F14_Method,
        "monitor": monitor.F10_Monitor,
        "multipath": multipath.FC6_MultiPath,
        "network": network.F9_Network,
        "nfs": nfs.FC6_NFS,
        "part": partition.F14_Partition,
        "partition": partition.F14_Partition,
        "poweroff": reboot.FC6_Reboot,
        "raid": raid.F15_Raid,
        "reboot": reboot.FC6_Reboot,
        "repo": repo.F15_Repo,
        "rescue": rescue.F10_Rescue,
        "rootpw": rootpw.F8_RootPw,
        "selinux": selinux.FC3_SELinux,
        "services": services.FC6_Services,
        "shutdown": reboot.FC6_Reboot,
        "skipx": skipx.FC3_SkipX,
        "sshpw": sshpw.F13_SshPw,
        "text": displaymode.FC3_DisplayMode,
        "timezone": timezone.FC6_Timezone,
        "updates": updates.F7_Updates,
        "upgrade": upgrade.F11_Upgrade,
        "url": url.F14_Url,
        "user": user.F12_User,
        "vnc": vnc.F9_Vnc,
        "volgroup": volgroup.FC3_VolGroup,
        "xconfig": xconfig.F14_XConfig,
        "zerombr": zerombr.F9_ZeroMbr,
        "zfcp": zfcp.F14_ZFCP,
    },

    # based on f15
    F16: {
        "auth": authconfig.FC3_Authconfig,
        "authconfig": authconfig.FC3_Authconfig,
        "autopart": autopart.F16_AutoPart,
        "autostep": autostep.FC3_AutoStep,
        "bootloader": bootloader.F15_Bootloader,
        "cdrom": cdrom.FC3_Cdrom,
        "clearpart": clearpart.FC3_ClearPart,
        "cmdline": displaymode.FC3_DisplayMode,
        "device": device.F8_Device,
        "deviceprobe": deviceprobe.FC3_DeviceProbe,
        "dmraid": dmraid.FC6_DmRaid,
        "driverdisk": driverdisk.F14_DriverDisk,
        "fcoe": fcoe.F13_Fcoe,
        "firewall": firewall.F14_Firewall,
        "firstboot": firstboot.FC3_Firstboot,
        "graphical": displaymode.FC3_DisplayMode,
        "group": group.F12_Group,
        "halt": reboot.FC6_Reboot,
        "harddrive": harddrive.FC3_HardDrive,
        "ignoredisk": ignoredisk.F14_IgnoreDisk,
        "install": upgrade.F11_Upgrade,
        "iscsi": iscsi.F10_Iscsi,
        "iscsiname": iscsiname.FC6_IscsiName,
        "keyboard": keyboard.FC3_Keyboard,
        "lang": lang.FC3_Lang,
        "logging": logging.FC6_Logging,
        "logvol": logvol.F15_LogVol,
        "mediacheck": mediacheck.FC4_MediaCheck,
        "method": method.F14_Method,
        "monitor": monitor.F10_Monitor,
        "multipath": multipath.FC6_MultiPath,
        "network": network.F16_Network,
        "nfs": nfs.FC6_NFS,
        "part": partition.F14_Partition,
        "partition": partition.F14_Partition,
        "poweroff": reboot.FC6_Reboot,
        "raid": raid.F19_Raid,
        "reboot": reboot.FC6_Reboot,
        "repo": repo.F15_Repo,
        "rescue": rescue.F10_Rescue,
        "rootpw": rootpw.F8_RootPw,
        "selinux": selinux.FC3_SELinux,
        "services": services.FC6_Services,
        "shutdown": reboot.FC6_Reboot,
        "skipx": skipx.FC3_SkipX,
        "sshpw": sshpw.F13_SshPw,
        "text": displaymode.FC3_DisplayMode,
        "timezone": timezone.FC6_Timezone,
        "updates": updates.F7_Updates,
        "upgrade": upgrade.F11_Upgrade,
        "url": url.F14_Url,
        "user": user.F12_User,
        "vnc": vnc.F9_Vnc,
        "volgroup": volgroup.FC16_VolGroup,
        "xconfig": xconfig.F14_XConfig,
        "zerombr": zerombr.F9_ZeroMbr,
        "zfcp": zfcp.F14_ZFCP,
    },

    # based on f16
    F17: {
        "auth": authconfig.FC3_Authconfig,
        "authconfig": authconfig.FC3_Authconfig,
        "autopart": autopart.F17_AutoPart,
        "autostep": autostep.FC3_AutoStep,
        "bootloader": bootloader.F17_Bootloader,
        "btrfs": btrfs.F17_BTRFS,
        "cdrom": cdrom.FC3_Cdrom,
        "clearpart": clearpart.F17_ClearPart,
        "cmdline": displaymode.FC3_DisplayMode,
        "device": device.F8_Device,
        "deviceprobe": deviceprobe.FC3_DeviceProbe,
        "dmraid": dmraid.FC6_DmRaid,
        "driverdisk": driverdisk.F14_DriverDisk,
        "fcoe": fcoe.F13_Fcoe,
        "firewall": firewall.F14_Firewall,
        "firstboot": firstboot.FC3_Firstboot,
        "graphical": displaymode.FC3_DisplayMode,
        "group": group.F12_Group,
        "halt": reboot.FC6_Reboot,
        "harddrive": harddrive.FC3_HardDrive,
        "ignoredisk": ignoredisk.F14_IgnoreDisk,
        "install": upgrade.F11_Upgrade,
        "iscsi": iscsi.F17_Iscsi,
        "iscsiname": iscsiname.FC6_IscsiName,
        "keyboard": keyboard.FC3_Keyboard,
        "lang": lang.FC3_Lang,
        "logging": logging.FC6_Logging,
        "logvol": logvol.F17_LogVol,
        "mediacheck": mediacheck.FC4_MediaCheck,
        "method": method.F14_Method,
        "monitor": monitor.F10_Monitor,
        "multipath": multipath.FC6_MultiPath,
        "network": network.F16_Network,
        "nfs": nfs.FC6_NFS,
        "part": partition.F17_Partition,
        "partition": partition.F17_Partition,
        "poweroff": reboot.FC6_Reboot,
        "raid": raid.F15_Raid,
        "reboot": reboot.FC6_Reboot,
        "repo": repo.F15_Repo,
        "rescue": rescue.F10_Rescue,
        "rootpw": rootpw.F8_RootPw,
        "selinux": selinux.FC3_SELinux,
        "services": services.FC6_Services,
        "shutdown": reboot.FC6_Reboot,
        "skipx": skipx.FC3_SkipX,
        "sshpw": sshpw.F13_SshPw,
        "text": displaymode.FC3_DisplayMode,
        "timezone": timezone.FC6_Timezone,
        "updates": updates.F7_Updates,
        "upgrade": upgrade.F11_Upgrade,
        "url": url.F14_Url,
        "user": user.F12_User,
        "vnc": vnc.F9_Vnc,
        "volgroup": volgroup.FC16_VolGroup,
        "xconfig": xconfig.F14_XConfig,
        "zerombr": zerombr.F9_ZeroMbr,
        "zfcp": zfcp.F14_ZFCP,
    },

    # based on f17
    F18: {
        "auth": authconfig.FC3_Authconfig,
        "authconfig": authconfig.FC3_Authconfig,
        "autopart": autopart.F18_AutoPart,
        "autostep": autostep.FC3_AutoStep,
        "bootloader": bootloader.F18_Bootloader,
        "btrfs": btrfs.F17_BTRFS,
        "cdrom": cdrom.FC3_Cdrom,
        "clearpart": clearpart.F17_ClearPart,
        "cmdline": displaymode.FC3_DisplayMode,
        "device": device.F8_Device,
        "deviceprobe": deviceprobe.FC3_DeviceProbe,
        "dmraid": dmraid.FC6_DmRaid,
        "driverdisk": driverdisk.F14_DriverDisk,
        "fcoe": fcoe.F13_Fcoe,
        "firewall": firewall.F14_Firewall,
        "firstboot": firstboot.FC3_Firstboot,
        "graphical": displaymode.FC3_DisplayMode,
        "group": group.F12_Group,
        "halt": reboot.F18_Reboot,
        "harddrive": harddrive.FC3_HardDrive,
        "ignoredisk": ignoredisk.F14_IgnoreDisk,
        "install": upgrade.F11_Upgrade,
        "iscsi": iscsi.F17_Iscsi,
        "iscsiname": iscsiname.FC6_IscsiName,
        "keyboard": keyboard.F18_Keyboard,
        "lang": lang.FC3_Lang,
        "logging": logging.FC6_Logging,
        "logvol": logvol.F18_LogVol,
        "mediacheck": mediacheck.FC4_MediaCheck,
        "method": method.F18_Method,
        "multipath": multipath.FC6_MultiPath,
        "network": network.F19_Network,
        "nfs": nfs.FC6_NFS,
        "part": partition.F18_Partition,
        "partition": partition.F18_Partition,
        "poweroff": reboot.F18_Reboot,
        "raid": raid.F18_Raid,
        "reboot": reboot.F18_Reboot,
        "repo": repo.F15_Repo,
        "rescue": rescue.F10_Rescue,
        "rootpw": rootpw.F18_RootPw,
        "selinux": selinux.FC3_SELinux,
        "services": services.FC6_Services,
        "shutdown": reboot.F18_Reboot,
        "skipx": skipx.FC3_SkipX,
        "sshpw": sshpw.F13_SshPw,
        "text": displaymode.FC3_DisplayMode,
        "timezone": timezone.F18_Timezone,
        "updates": updates.F7_Updates,
        "upgrade": upgrade.F11_Upgrade,
        "url": url.F18_Url,
        "user": user.F12_User,
        "vnc": vnc.F9_Vnc,
        "volgroup": volgroup.FC16_VolGroup,
        "xconfig": xconfig.F14_XConfig,
        "zerombr": zerombr.F9_ZeroMbr,
        "zfcp": zfcp.F14_ZFCP,
    },

    # based on f18
    F19: {
        "auth": authconfig.FC3_Authconfig,
        "authconfig": authconfig.FC3_Authconfig,
        "autopart": autopart.F18_AutoPart,
        "autostep": autostep.FC3_AutoStep,
        "bootloader": bootloader.F19_Bootloader,
        "btrfs": btrfs.F17_BTRFS,
        "cdrom": cdrom.FC3_Cdrom,
        "clearpart": clearpart.F17_ClearPart,
        "cmdline": displaymode.FC3_DisplayMode,
        "device": device.F8_Device,
        "deviceprobe": deviceprobe.FC3_DeviceProbe,
        "dmraid": dmraid.FC6_DmRaid,
        "driverdisk": driverdisk.F14_DriverDisk,
        "fcoe": fcoe.F13_Fcoe,
        "firewall": firewall.F14_Firewall,
        "firstboot": firstboot.FC3_Firstboot,
        "graphical": displaymode.FC3_DisplayMode,
        "group": group.F12_Group,
        "halt": reboot.F18_Reboot,
        "harddrive": harddrive.FC3_HardDrive,
        "ignoredisk": ignoredisk.F14_IgnoreDisk,
        "install": upgrade.F11_Upgrade,
        "iscsi": iscsi.F17_Iscsi,
        "iscsiname": iscsiname.FC6_IscsiName,
        "keyboard": keyboard.F18_Keyboard,
        "lang": lang.F19_Lang,
        "liveimg": liveimg.F19_Liveimg,
        "logging": logging.FC6_Logging,
        "logvol": logvol.F18_LogVol,
        "mediacheck": mediacheck.FC4_MediaCheck,
        "method": method.F19_Method,
        "multipath": multipath.FC6_MultiPath,
        "network": network.F19_Network,
        "nfs": nfs.FC6_NFS,
        "part": partition.F18_Partition,
        "partition": partition.F18_Partition,
        "poweroff": reboot.F18_Reboot,
        "raid": raid.F19_Raid,
        "realm": realm.F19_Realm,
        "reboot": reboot.F18_Reboot,
        "repo": repo.F15_Repo,
        "rescue": rescue.F10_Rescue,
        "rootpw": rootpw.F18_RootPw,
        "selinux": selinux.FC3_SELinux,
        "services": services.FC6_Services,
        "shutdown": reboot.F18_Reboot,
        "skipx": skipx.FC3_SkipX,
        "sshpw": sshpw.F13_SshPw,
        "text": displaymode.FC3_DisplayMode,
        "timezone": timezone.F18_Timezone,
        "updates": updates.F7_Updates,
        "upgrade": upgrade.F11_Upgrade,
        "url": url.F18_Url,
        "user": user.F19_User,
        "vnc": vnc.F9_Vnc,
        "volgroup": volgroup.FC16_VolGroup,
        "xconfig": xconfig.F14_XConfig,
        "zerombr": zerombr.F9_ZeroMbr,
        "zfcp": zfcp.F14_ZFCP,
    },

    # based on f19
    F20: {
        "auth": authconfig.FC3_Authconfig,
        "authconfig": authconfig.FC3_Authconfig,
        "autopart": autopart.F20_AutoPart,
        "autostep": autostep.FC3_AutoStep,
        "bootloader": bootloader.F19_Bootloader,
        "btrfs": btrfs.F17_BTRFS,
        "cdrom": cdrom.FC3_Cdrom,
        "clearpart": clearpart.F17_ClearPart,
        "cmdline": displaymode.FC3_DisplayMode,
        "device": device.F8_Device,
        "deviceprobe": deviceprobe.FC3_DeviceProbe,
        "dmraid": dmraid.FC6_DmRaid,
        "driverdisk": driverdisk.F14_DriverDisk,
        "eula": eula.F20_Eula,
        "fcoe": fcoe.F13_Fcoe,
        "firewall": firewall.F20_Firewall,
        "firstboot": firstboot.FC3_Firstboot,
        "graphical": displaymode.FC3_DisplayMode,
        "group": group.F12_Group,
        "halt": reboot.F18_Reboot,
        "harddrive": harddrive.FC3_HardDrive,
        "ignoredisk": ignoredisk.F14_IgnoreDisk,
        "install": upgrade.F11_Upgrade,
        "iscsi": iscsi.F17_Iscsi,
        "iscsiname": iscsiname.FC6_IscsiName,
        "keyboard": keyboard.F18_Keyboard,
        "lang": lang.F19_Lang,
        "liveimg": liveimg.F19_Liveimg,
        "logging": logging.FC6_Logging,
        "logvol": logvol.F20_LogVol,
        "mediacheck": mediacheck.FC4_MediaCheck,
        "method": method.F19_Method,
        "multipath": multipath.FC6_MultiPath,
        "network": network.F19_Network,
        "nfs": nfs.FC6_NFS,
        "part": partition.F20_Partition,
        "partition": partition.F20_Partition,
        "poweroff": reboot.F18_Reboot,
        "raid": raid.F20_Raid,
        "realm": realm.F19_Realm,
        "reboot": reboot.F18_Reboot,
        "repo": repo.F15_Repo,
        "rescue": rescue.F10_Rescue,
        "rootpw": rootpw.F18_RootPw,
        "selinux": selinux.FC3_SELinux,
        "services": services.FC6_Services,
        "shutdown": reboot.F18_Reboot,
        "skipx": skipx.FC3_SkipX,
        "sshpw": sshpw.F13_SshPw,
        "text": displaymode.FC3_DisplayMode,
        "timezone": timezone.F18_Timezone,
        "updates": updates.F7_Updates,
        "upgrade": upgrade.F20_Upgrade,
        "url": url.F18_Url,
        "user": user.F19_User,
        "vnc": vnc.F9_Vnc,
        "volgroup": volgroup.F20_VolGroup,
        "xconfig": xconfig.F14_XConfig,
        "zerombr": zerombr.F9_ZeroMbr,
        "zfcp": zfcp.F14_ZFCP,
    },

    # based on fc1
    RHEL3: {
        "auth": authconfig.FC3_Authconfig,
        "authconfig": authconfig.FC3_Authconfig,
        "autopart": autopart.FC3_AutoPart,
        "autostep": autostep.FC3_AutoStep,
        "bootloader": bootloader.FC3_Bootloader,
        "cdrom": cdrom.FC3_Cdrom,
        "clearpart": clearpart.FC3_ClearPart,
        "cmdline": displaymode.FC3_DisplayMode,
        "device": device.FC3_Device,
        "deviceprobe": deviceprobe.FC3_DeviceProbe,
        "driverdisk": driverdisk.FC3_DriverDisk,
        "firewall": firewall.FC3_Firewall,
        "firstboot": firstboot.FC3_Firstboot,
        "graphical": displaymode.FC3_DisplayMode,
        "halt": reboot.FC3_Reboot,
        "harddrive": harddrive.FC3_HardDrive,
        "ignoredisk": ignoredisk.FC3_IgnoreDisk,
        "install": upgrade.FC3_Upgrade,
        "interactive": interactive.FC3_Interactive,
        "keyboard": keyboard.FC3_Keyboard,
        "lang": lang.FC3_Lang,
        "langsupport": langsupport.FC3_LangSupport,
        "lilo": bootloader.FC3_Bootloader,
        "lilocheck": lilocheck.FC3_LiloCheck,
        "logvol": logvol.FC3_LogVol,
        "method": method.FC3_Method,
        "monitor": monitor.FC3_Monitor,
        "mouse": mouse.RHEL3_Mouse,
        "network": network.FC3_Network,
        "nfs": nfs.FC3_NFS,
        "part": partition.FC3_Partition,
        "partition": partition.FC3_Partition,
        "poweroff": reboot.FC3_Reboot,
        "raid": raid.FC3_Raid,
        "reboot": reboot.FC3_Reboot,
        "rootpw": rootpw.FC3_RootPw,
        "shutdown": reboot.FC3_Reboot,
        "skipx": skipx.FC3_SkipX,
        "text": displaymode.FC3_DisplayMode,
        "timezone": timezone.FC3_Timezone,
        "upgrade": upgrade.FC3_Upgrade,
        "url": url.FC3_Url,
        "vnc": vnc.FC3_Vnc,
        "volgroup": volgroup.FC3_VolGroup,
        "xconfig": xconfig.FC3_XConfig,
        "zerombr": zerombr.FC3_ZeroMbr,
    },

    # based on fc3
    RHEL4: {
        "auth": authconfig.FC3_Authconfig,
        "authconfig": authconfig.FC3_Authconfig,
        "autopart": autopart.FC3_AutoPart,
        "autostep": autostep.FC3_AutoStep,
        "bootloader": bootloader.FC3_Bootloader,
        "cdrom": cdrom.FC3_Cdrom,
        "clearpart": clearpart.FC3_ClearPart,
        "cmdline": displaymode.FC3_DisplayMode,
        "device": device.FC3_Device,
        "deviceprobe": deviceprobe.FC3_DeviceProbe,
        "driverdisk": driverdisk.FC4_DriverDisk,
        "firewall": firewall.FC3_Firewall,
        "firstboot": firstboot.FC3_Firstboot,
        "graphical": displaymode.FC3_DisplayMode,
        "halt": reboot.FC3_Reboot,
        "harddrive": harddrive.FC3_HardDrive,
        "ignoredisk": ignoredisk.F8_IgnoreDisk,
        "install": upgrade.FC3_Upgrade,
        "interactive": interactive.FC3_Interactive,
        "keyboard": keyboard.FC3_Keyboard,
        "lang": lang.FC3_Lang,
        "langsupport": langsupport.FC3_LangSupport,
        "lilo": bootloader.FC3_Bootloader,
        "lilocheck": lilocheck.FC3_LiloCheck,
        "logvol": logvol.FC3_LogVol,
        "method": method.FC3_Method,
        "monitor": monitor.FC3_Monitor,
        "mouse": mouse.FC3_Mouse,
        "network": network.RHEL4_Network,
        "nfs": nfs.FC3_NFS,
        "part": partition.FC3_Partition,
        "partition": partition.FC3_Partition,
        "poweroff": reboot.FC3_Reboot,
        "raid": raid.FC3_Raid,
        "reboot": reboot.FC3_Reboot,
        "rootpw": rootpw.FC3_RootPw,
        "selinux": selinux.FC3_SELinux,
        "shutdown": reboot.FC3_Reboot,
        "skipx": skipx.FC3_SkipX,
        "text": displaymode.FC3_DisplayMode,
        "timezone": timezone.FC3_Timezone,
        "upgrade": upgrade.FC3_Upgrade,
        "url": url.FC3_Url,
        "vnc": vnc.FC3_Vnc,
        "volgroup": volgroup.FC3_VolGroup,
        "xconfig": xconfig.FC3_XConfig,
        "zerombr": zerombr.FC3_ZeroMbr,
        "zfcp": zfcp.FC3_ZFCP,
    },

    # based on fc6
    RHEL5: {
        "auth": authconfig.FC3_Authconfig,
        "authconfig": authconfig.FC3_Authconfig,
        "autopart": autopart.F9_AutoPart,
        "autostep": autostep.FC3_AutoStep,
        "bootloader": bootloader.RHEL5_Bootloader,
        "cdrom": cdrom.FC3_Cdrom,
        "clearpart": clearpart.FC3_ClearPart,
        "cmdline": displaymode.FC3_DisplayMode,
        "device": device.FC3_Device,
        "deviceprobe": deviceprobe.FC3_DeviceProbe,
        "dmraid": dmraid.FC6_DmRaid,
        "driverdisk": driverdisk.F12_DriverDisk,
        "firewall": firewall.FC3_Firewall,
        "firstboot": firstboot.FC3_Firstboot,
        "graphical": displaymode.FC3_DisplayMode,
        "halt": reboot.FC6_Reboot,
        "harddrive": harddrive.FC3_HardDrive,
        "ignoredisk": ignoredisk.F8_IgnoreDisk,
        "install": upgrade.FC3_Upgrade,
        "interactive": interactive.FC3_Interactive,
        "iscsi": iscsi.FC6_Iscsi,
        "iscsiname": iscsiname.FC6_IscsiName,
        "key": key.RHEL5_Key,
        "keyboard": keyboard.FC3_Keyboard,
        "lang": lang.FC3_Lang,
        "langsupport": langsupport.FC5_LangSupport,
        "logging": logging.FC6_Logging,
        "logvol": logvol.RHEL5_LogVol,
        "mediacheck": mediacheck.FC4_MediaCheck,
        "method": method.FC6_Method,
        "monitor": monitor.FC6_Monitor,
        "mouse": mouse.FC3_Mouse,
        "multipath": multipath.FC6_MultiPath,
        "network": network.RHEL5_Network,
        "nfs": nfs.FC6_NFS,
        "part": partition.RHEL5_Partition,
        "partition": partition.RHEL5_Partition,
        "poweroff": reboot.FC6_Reboot,
        "raid": raid.RHEL5_Raid,
        "reboot": reboot.FC6_Reboot,
        "repo": repo.FC6_Repo,
        "rootpw": rootpw.FC3_RootPw,
        "services": services.FC6_Services,
        "selinux": selinux.FC3_SELinux,
        "shutdown": reboot.FC6_Reboot,
        "skipx": skipx.FC3_SkipX,
        "text": displaymode.FC3_DisplayMode,
        "timezone": timezone.FC6_Timezone,
        "upgrade": upgrade.FC3_Upgrade,
        "user": user.FC6_User,
        "url": url.FC3_Url,
        "vnc": vnc.FC6_Vnc,
        "volgroup": volgroup.FC3_VolGroup,
        "xconfig": xconfig.FC6_XConfig,
        "zerombr": zerombr.FC3_ZeroMbr,
        "zfcp": zfcp.FC3_ZFCP,
    },

    # based on f13ish
    RHEL6: {
        "auth": authconfig.FC3_Authconfig,
        "authconfig": authconfig.FC3_Authconfig,
        "autopart": autopart.RHEL6_AutoPart,
        "autostep": autostep.FC3_AutoStep,
        "bootloader": bootloader.RHEL6_Bootloader,
        "cdrom": cdrom.FC3_Cdrom,
        "clearpart": clearpart.FC3_ClearPart,
        "cmdline": displaymode.FC3_DisplayMode,
        "device": device.F8_Device,
        "deviceprobe": deviceprobe.FC3_DeviceProbe,
        "dmraid": dmraid.FC6_DmRaid,
        "driverdisk": driverdisk.F12_DriverDisk,
        "fcoe": fcoe.F13_Fcoe,
        "firewall": firewall.F10_Firewall,
        "firstboot": firstboot.FC3_Firstboot,
        "graphical": displaymode.FC3_DisplayMode,
        "group": group.F12_Group,
        "halt": reboot.FC6_Reboot,
        "harddrive": harddrive.FC3_HardDrive,
        "ignoredisk": ignoredisk.RHEL6_IgnoreDisk,
        "install": upgrade.F11_Upgrade,
        "interactive": interactive.FC3_Interactive,
        "iscsi": iscsi.RHEL6_Iscsi,
        "iscsiname": iscsiname.FC6_IscsiName,
        "keyboard": keyboard.FC3_Keyboard,
        "lang": lang.FC3_Lang,
        "logging": logging.FC6_Logging,
        "logvol": logvol.RHEL6_LogVol,
        "mediacheck": mediacheck.FC4_MediaCheck,
        "method": method.F13_Method,
        "monitor": monitor.F10_Monitor,
        "multipath": multipath.FC6_MultiPath,
        "network": network.RHEL6_Network,
        "nfs": nfs.FC6_NFS,
        "part": partition.RHEL6_Partition,
        "partition": partition.RHEL6_Partition,
        "poweroff": reboot.FC6_Reboot,
        "raid": raid.RHEL6_Raid,
        "reboot": reboot.FC6_Reboot,
        "repo": repo.RHEL6_Repo,
        "rescue": rescue.F10_Rescue,
        "rootpw": rootpw.F8_RootPw,
        "selinux": selinux.FC3_SELinux,
        "services": services.FC6_Services,
        "shutdown": reboot.FC6_Reboot,
        "skipx": skipx.FC3_SkipX,
        "sshpw": sshpw.F13_SshPw,
        "text": displaymode.FC3_DisplayMode,
        "timezone": timezone.FC6_Timezone,
        "unsupported_hardware": unsupported_hardware.RHEL6_UnsupportedHardware,
        "updates": updates.F7_Updates,
        "upgrade": upgrade.F11_Upgrade,
        "url": url.RHEL6_Url,
        "user": user.F12_User,
        "vnc": vnc.F9_Vnc,
        "volgroup": volgroup.RHEL6_VolGroup,
        "xconfig": xconfig.F10_XConfig,
        "zerombr": zerombr.F9_ZeroMbr,
        "zfcp": zfcp.F12_ZFCP,
    },

    # based on f19
    RHEL7: {
        "auth": authconfig.FC3_Authconfig,
        "authconfig": authconfig.FC3_Authconfig,
        "autopart": autopart.F20_AutoPart,
        "autostep": autostep.FC3_AutoStep,
        "bootloader": bootloader.F19_Bootloader,
        "btrfs": btrfs.F17_BTRFS,
        "cdrom": cdrom.FC3_Cdrom,
        "clearpart": clearpart.F17_ClearPart,
        "cmdline": displaymode.FC3_DisplayMode,
        "device": device.F8_Device,
        "deviceprobe": deviceprobe.FC3_DeviceProbe,
        "dmraid": dmraid.FC6_DmRaid,
        "driverdisk": driverdisk.F14_DriverDisk,
        "eula": eula.F20_Eula,
        "fcoe": fcoe.F13_Fcoe,
        "firewall": firewall.F14_Firewall,
        "firstboot": firstboot.FC3_Firstboot,
        "graphical": displaymode.FC3_DisplayMode,
        "group": group.F12_Group,
        "halt": reboot.F18_Reboot,
        "harddrive": harddrive.FC3_HardDrive,
        "ignoredisk": ignoredisk.F14_IgnoreDisk,
        "install": upgrade.F11_Upgrade,
        "iscsi": iscsi.F17_Iscsi,
        "iscsiname": iscsiname.FC6_IscsiName,
        "keyboard": keyboard.F18_Keyboard,
        "lang": lang.F19_Lang,
        "liveimg": liveimg.F19_Liveimg,
        "logging": logging.FC6_Logging,
        "logvol": logvol.F20_LogVol,
        "mediacheck": mediacheck.FC4_MediaCheck,
        "method": method.F19_Method,
        "multipath": multipath.FC6_MultiPath,
        "network": network.F19_Network,
        "nfs": nfs.FC6_NFS,
        "part": partition.F20_Partition,
        "partition": partition.F20_Partition,
        "poweroff": reboot.F18_Reboot,
        "raid": raid.F20_Raid,
        "realm": realm.F19_Realm,
        "reboot": reboot.F18_Reboot,
        "repo": repo.F15_Repo,
        "rescue": rescue.F10_Rescue,
        "rootpw": rootpw.F18_RootPw,
        "selinux": selinux.FC3_SELinux,
        "services": services.FC6_Services,
        "shutdown": reboot.F18_Reboot,
        "skipx": skipx.FC3_SkipX,
        "sshpw": sshpw.F13_SshPw,
        "text": displaymode.FC3_DisplayMode,
        "timezone": timezone.F18_Timezone,
        "unsupported_hardware": unsupported_hardware.RHEL6_UnsupportedHardware,
        "updates": updates.F7_Updates,
        "upgrade": upgrade.F20_Upgrade,
        "url": url.F18_Url,
        "user": user.F19_User,
        "vnc": vnc.F9_Vnc,
        "volgroup": volgroup.FC16_VolGroup,
        "xconfig": xconfig.F14_XConfig,
        "zerombr": zerombr.F9_ZeroMbr,
        "zfcp": zfcp.F14_ZFCP,
    }
}

# This map is keyed on kickstart syntax version as provided by
# pykickstart.version.  Within each sub-dict is a mapping from a data object
# name to the class that provides it.  This is a bijective mapping - that is,
# each name maps to exactly one data class and all data classes have a name.
# More than one instance of each class is allowed to exist, however.
dataMap = {
    FC3: {
        "DriverDiskData": driverdisk.FC3_DriverDiskData,
        "LogVolData": logvol.FC3_LogVolData,
        "NetworkData": network.FC3_NetworkData,
        "PartData": partition.FC3_PartData,
        "RaidData": raid.FC3_RaidData,
        "VolGroupData": volgroup.FC3_VolGroupData,
        "ZFCPData": zfcp.FC3_ZFCPData,
    },
    FC4: {
        "DriverDiskData": driverdisk.FC4_DriverDiskData,
        "LogVolData": logvol.FC4_LogVolData,
        "NetworkData": network.FC4_NetworkData,
        "PartData": partition.FC4_PartData,
        "RaidData": raid.FC4_RaidData,
        "VolGroupData": volgroup.FC3_VolGroupData,
        "ZFCPData": zfcp.FC3_ZFCPData,
    },
    FC5: {
        "DriverDiskData": driverdisk.FC4_DriverDiskData,
        "LogVolData": logvol.FC4_LogVolData,
        "NetworkData": network.FC4_NetworkData,
        "PartData": partition.FC4_PartData,
        "RaidData": raid.FC5_RaidData,
        "VolGroupData": volgroup.FC3_VolGroupData,
        "ZFCPData": zfcp.FC3_ZFCPData,
    },
    FC6: {
        "DriverDiskData": driverdisk.FC4_DriverDiskData,
        "DmRaidData": dmraid.FC6_DmRaidData,
        "IscsiData": iscsi.FC6_IscsiData,
        "LogVolData": logvol.FC4_LogVolData,
        "MultiPathData": multipath.FC6_MultiPathData,
        "NetworkData": network.FC6_NetworkData,
        "PartData": partition.FC4_PartData,
        "RaidData": raid.FC5_RaidData,
        "RepoData": repo.FC6_RepoData,
        "UserData": user.FC6_UserData,
        "VolGroupData": volgroup.FC3_VolGroupData,
        "ZFCPData": zfcp.FC3_ZFCPData,
    },
    F7: {
        "DriverDiskData": driverdisk.FC4_DriverDiskData,
        "DmRaidData": dmraid.FC6_DmRaidData,
        "IscsiData": iscsi.FC6_IscsiData,
        "LogVolData": logvol.FC4_LogVolData,
        "MultiPathData": multipath.FC6_MultiPathData,
        "NetworkData": network.FC6_NetworkData,
        "PartData": partition.FC4_PartData,
        "RaidData": raid.F7_RaidData,
        "RepoData": repo.FC6_RepoData,
        "UserData": user.FC6_UserData,
        "VolGroupData": volgroup.FC3_VolGroupData,
        "ZFCPData": zfcp.FC3_ZFCPData,
    },
    F8: {
        "DriverDiskData": driverdisk.FC4_DriverDiskData,
        "DeviceData": device.F8_DeviceData,
        "DmRaidData": dmraid.FC6_DmRaidData,
        "IscsiData": iscsi.FC6_IscsiData,
        "LogVolData": logvol.FC4_LogVolData,
        "MultiPathData": multipath.FC6_MultiPathData,
        "NetworkData": network.F8_NetworkData,
        "PartData": partition.FC4_PartData,
        "RaidData": raid.F7_RaidData,
        "RepoData": repo.F8_RepoData,
        "UserData": user.F8_UserData,
        "VolGroupData": volgroup.FC3_VolGroupData,
        "ZFCPData": zfcp.FC3_ZFCPData,
    },
    F9: {
        "DriverDiskData": driverdisk.FC4_DriverDiskData,
        "DeviceData": device.F8_DeviceData,
        "DmRaidData": dmraid.FC6_DmRaidData,
        "IscsiData": iscsi.FC6_IscsiData,
        "LogVolData": logvol.F9_LogVolData,
        "MultiPathData": multipath.FC6_MultiPathData,
        "NetworkData": network.F8_NetworkData,
        "PartData": partition.F9_PartData,
        "RaidData": raid.F9_RaidData,
        "RepoData": repo.F8_RepoData,
        "UserData": user.F8_UserData,
        "VolGroupData": volgroup.FC3_VolGroupData,
        "ZFCPData": zfcp.FC3_ZFCPData,
    },
    F10: {
        "DriverDiskData": driverdisk.FC4_DriverDiskData,
        "DeviceData": device.F8_DeviceData,
        "DmRaidData": dmraid.FC6_DmRaidData,
        "IscsiData": iscsi.F10_IscsiData,
        "LogVolData": logvol.F9_LogVolData,
        "MultiPathData": multipath.FC6_MultiPathData,
        "NetworkData": network.F8_NetworkData,
        "PartData": partition.F9_PartData,
        "RaidData": raid.F9_RaidData,
        "RepoData": repo.F8_RepoData,
        "UserData": user.F8_UserData,
        "VolGroupData": volgroup.FC3_VolGroupData,
        "ZFCPData": zfcp.FC3_ZFCPData,
    },
    F11: {
        "DriverDiskData": driverdisk.FC4_DriverDiskData,
        "DeviceData": device.F8_DeviceData,
        "DmRaidData": dmraid.FC6_DmRaidData,
        "IscsiData": iscsi.F10_IscsiData,
        "LogVolData": logvol.F9_LogVolData,
        "MultiPathData": multipath.FC6_MultiPathData,
        "NetworkData": network.F8_NetworkData,
        "PartData": partition.F11_PartData,
        "RaidData": raid.F9_RaidData,
        "RepoData": repo.F11_RepoData,
        "UserData": user.F8_UserData,
        "VolGroupData": volgroup.FC3_VolGroupData,
        "ZFCPData": zfcp.FC3_ZFCPData,
    },
    F12: {
        "DriverDiskData": driverdisk.F12_DriverDiskData,
        "DeviceData": device.F8_DeviceData,
        "DmRaidData": dmraid.FC6_DmRaidData,
        "FcoeData": fcoe.F12_FcoeData,
        "GroupData": group.F12_GroupData,
        "IscsiData": iscsi.F10_IscsiData,
        "LogVolData": logvol.F12_LogVolData,
        "MultiPathData": multipath.FC6_MultiPathData,
        "NetworkData": network.F8_NetworkData,
        "PartData": partition.F12_PartData,
        "RaidData": raid.F12_RaidData,
        "RepoData": repo.F11_RepoData,
        "UserData": user.F12_UserData,
        "VolGroupData": volgroup.FC3_VolGroupData,
        "ZFCPData": zfcp.F12_ZFCPData,
    },
    F13: {
        "DriverDiskData": driverdisk.F12_DriverDiskData,
        "DeviceData": device.F8_DeviceData,
        "DmRaidData": dmraid.FC6_DmRaidData,
        "FcoeData": fcoe.F13_FcoeData,
        "GroupData": group.F12_GroupData,
        "IscsiData": iscsi.F10_IscsiData,
        "LogVolData": logvol.F12_LogVolData,
        "MultiPathData": multipath.FC6_MultiPathData,
        "NetworkData": network.F8_NetworkData,
        "PartData": partition.F12_PartData,
        "RaidData": raid.F13_RaidData,
        "RepoData": repo.F13_RepoData,
        "SshPwData": sshpw.F13_SshPwData,
        "UserData": user.F12_UserData,
        "VolGroupData": volgroup.FC3_VolGroupData,
        "ZFCPData": zfcp.F12_ZFCPData,
    },
    F14: {
        "DriverDiskData": driverdisk.F14_DriverDiskData,
        "DeviceData": device.F8_DeviceData,
        "DmRaidData": dmraid.FC6_DmRaidData,
        "FcoeData": fcoe.F13_FcoeData,
        "GroupData": group.F12_GroupData,
        "IscsiData": iscsi.F10_IscsiData,
        "LogVolData": logvol.F14_LogVolData,
        "MultiPathData": multipath.FC6_MultiPathData,
        "NetworkData": network.F8_NetworkData,
        "PartData": partition.F14_PartData,
        "RaidData": raid.F14_RaidData,
        "RepoData": repo.F14_RepoData,
        "SshPwData": sshpw.F13_SshPwData,
        "UserData": user.F12_UserData,
        "VolGroupData": volgroup.FC3_VolGroupData,
        "ZFCPData": zfcp.F14_ZFCPData,
    },
    F15: {
        "DriverDiskData": driverdisk.F14_DriverDiskData,
        "DeviceData": device.F8_DeviceData,
        "DmRaidData": dmraid.FC6_DmRaidData,
        "FcoeData": fcoe.F13_FcoeData,
        "GroupData": group.F12_GroupData,
        "IscsiData": iscsi.F10_IscsiData,
        "LogVolData": logvol.F15_LogVolData,
        "MultiPathData": multipath.FC6_MultiPathData,
        "NetworkData": network.F8_NetworkData,
        "PartData": partition.F14_PartData,
        "RaidData": raid.F15_RaidData,
        "RepoData": repo.F15_RepoData,
        "SshPwData": sshpw.F13_SshPwData,
        "UserData": user.F12_UserData,
        "VolGroupData": volgroup.FC3_VolGroupData,
        "ZFCPData": zfcp.F14_ZFCPData,
    },
    F16: {
        "DriverDiskData": driverdisk.F14_DriverDiskData,
        "DeviceData": device.F8_DeviceData,
        "DmRaidData": dmraid.FC6_DmRaidData,
        "FcoeData": fcoe.F13_FcoeData,
        "GroupData": group.F12_GroupData,
        "IscsiData": iscsi.F10_IscsiData,
        "LogVolData": logvol.F17_LogVolData,
        "MultiPathData": multipath.FC6_MultiPathData,
        "NetworkData": network.F16_NetworkData,
        "PartData": partition.F17_PartData,
        "RaidData": raid.F15_RaidData,
        "RepoData": repo.F15_RepoData,
        "SshPwData": sshpw.F13_SshPwData,
        "UserData": user.F12_UserData,
        "VolGroupData": volgroup.FC16_VolGroupData,
        "ZFCPData": zfcp.F14_ZFCPData,
    },
    F17: {
        "BTRFSData": btrfs.F17_BTRFSData,
        "DriverDiskData": driverdisk.F14_DriverDiskData,
        "DeviceData": device.F8_DeviceData,
        "DmRaidData": dmraid.FC6_DmRaidData,
        "FcoeData": fcoe.F13_FcoeData,
        "GroupData": group.F12_GroupData,
        "IscsiData": iscsi.F17_IscsiData,
        "LogVolData": logvol.F17_LogVolData,
        "MultiPathData": multipath.FC6_MultiPathData,
        "NetworkData": network.F16_NetworkData,
        "PartData": partition.F17_PartData,
        "RaidData": raid.F15_RaidData,
        "RepoData": repo.F15_RepoData,
        "SshPwData": sshpw.F13_SshPwData,
        "UserData": user.F12_UserData,
        "VolGroupData": volgroup.FC16_VolGroupData,
        "ZFCPData": zfcp.F14_ZFCPData,
    },
    F18: {
        "BTRFSData": btrfs.F17_BTRFSData,
        "DriverDiskData": driverdisk.F14_DriverDiskData,
        "DeviceData": device.F8_DeviceData,
        "DmRaidData": dmraid.FC6_DmRaidData,
        "FcoeData": fcoe.F13_FcoeData,
        "GroupData": group.F12_GroupData,
        "IscsiData": iscsi.F17_IscsiData,
        "LogVolData": logvol.F18_LogVolData,
        "MultiPathData": multipath.FC6_MultiPathData,
        "NetworkData": network.F16_NetworkData,
        "PartData": partition.F18_PartData,
        "RaidData": raid.F18_RaidData,
        "RepoData": repo.F15_RepoData,
        "SshPwData": sshpw.F13_SshPwData,
        "UserData": user.F12_UserData,
        "VolGroupData": volgroup.FC16_VolGroupData,
        "ZFCPData": zfcp.F14_ZFCPData,
    },
    F19: {
        "BTRFSData": btrfs.F17_BTRFSData,
        "DriverDiskData": driverdisk.F14_DriverDiskData,
        "DeviceData": device.F8_DeviceData,
        "DmRaidData": dmraid.FC6_DmRaidData,
        "FcoeData": fcoe.F13_FcoeData,
        "GroupData": group.F12_GroupData,
        "IscsiData": iscsi.F17_IscsiData,
        "LogVolData": logvol.F20_LogVolData,
        "MultiPathData": multipath.FC6_MultiPathData,
        "NetworkData": network.F19_NetworkData,
        "PartData": partition.F18_PartData,
        "RaidData": raid.F18_RaidData,
        "RepoData": repo.F15_RepoData,
        "SshPwData": sshpw.F13_SshPwData,
        "UserData": user.F19_UserData,
        "VolGroupData": volgroup.FC16_VolGroupData,
        "ZFCPData": zfcp.F14_ZFCPData,
    },
    F20: {
        "BTRFSData": btrfs.F17_BTRFSData,
        "DriverDiskData": driverdisk.F14_DriverDiskData,
        "DeviceData": device.F8_DeviceData,
        "DmRaidData": dmraid.FC6_DmRaidData,
        "FcoeData": fcoe.F13_FcoeData,
        "GroupData": group.F12_GroupData,
        "IscsiData": iscsi.F17_IscsiData,
        "LogVolData": logvol.F20_LogVolData,
        "MultiPathData": multipath.FC6_MultiPathData,
        "NetworkData": network.F19_NetworkData,
        "PartData": partition.F18_PartData,
        "RaidData": raid.F18_RaidData,
        "RepoData": repo.F15_RepoData,
        "SshPwData": sshpw.F13_SshPwData,
        "UserData": user.F19_UserData,
        "VolGroupData": volgroup.FC16_VolGroupData,
        "ZFCPData": zfcp.F14_ZFCPData,
    },
    RHEL3: {
        "DriverDiskData": driverdisk.FC3_DriverDiskData,
        "LogVolData": logvol.FC3_LogVolData,
        "NetworkData": network.RHEL4_NetworkData,
        "PartData": partition.FC3_PartData,
        "RaidData": raid.FC3_RaidData,
        "VolGroupData": volgroup.FC3_VolGroupData,
        "ZFCPData": zfcp.FC3_ZFCPData,
    },
    RHEL4: {
        "DriverDiskData": driverdisk.FC4_DriverDiskData,
        "LogVolData": logvol.FC3_LogVolData,
        "NetworkData": network.RHEL4_NetworkData,
        "PartData": partition.FC3_PartData,
        "RaidData": raid.FC3_RaidData,
        "VolGroupData": volgroup.FC3_VolGroupData,
        "ZFCPData": zfcp.FC3_ZFCPData,
    },
    RHEL5: {
        "DriverDiskData": driverdisk.F12_DriverDiskData,
        "DmRaidData": dmraid.FC6_DmRaidData,
        "IscsiData": iscsi.FC6_IscsiData,
        "LogVolData": logvol.RHEL5_LogVolData,
        "MultiPathData": multipath.FC6_MultiPathData,
        "NetworkData": network.FC6_NetworkData,
        "PartData": partition.RHEL5_PartData,
        "RaidData": raid.RHEL5_RaidData,
        "RepoData": repo.FC6_RepoData,
        "UserData": user.FC6_UserData,
        "VolGroupData": volgroup.FC3_VolGroupData,
        "ZFCPData": zfcp.FC3_ZFCPData,
    },
    RHEL6: {
        "DriverDiskData": driverdisk.F12_DriverDiskData,
        "DeviceData": device.F8_DeviceData,
        "DmRaidData": dmraid.FC6_DmRaidData,
        "FcoeData": fcoe.F13_FcoeData,
        "GroupData": group.F12_GroupData,
        "IscsiData": iscsi.RHEL6_IscsiData,
        "LogVolData": logvol.RHEL6_LogVolData,
        "MultiPathData": multipath.FC6_MultiPathData,
        "NetworkData": network.RHEL6_NetworkData,
        "PartData": partition.RHEL6_PartData,
        "RaidData": raid.RHEL6_RaidData,
        "RepoData": repo.RHEL6_RepoData,
        "SshPwData": sshpw.F13_SshPwData,
        "UserData": user.F12_UserData,
        "VolGroupData": volgroup.FC3_VolGroupData,
        "ZFCPData": zfcp.F12_ZFCPData,
    },
    RHEL7: {
        "BTRFSData": btrfs.F17_BTRFSData,
        "DriverDiskData": driverdisk.F14_DriverDiskData,
        "DeviceData": device.F8_DeviceData,
        "DmRaidData": dmraid.FC6_DmRaidData,
        "FcoeData": fcoe.F13_FcoeData,
        "GroupData": group.F12_GroupData,
        "IscsiData": iscsi.F17_IscsiData,
        "LogVolData": logvol.F18_LogVolData,
        "MultiPathData": multipath.FC6_MultiPathData,
        "NetworkData": network.F19_NetworkData,
        "PartData": partition.F18_PartData,
        "RaidData": raid.F18_RaidData,
        "RepoData": repo.F15_RepoData,
        "SshPwData": sshpw.F13_SshPwData,
        "UserData": user.F19_UserData,
        "VolGroupData": volgroup.FC16_VolGroupData,
        "ZFCPData": zfcp.F14_ZFCPData,
    }
}



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2005, 2006, 2007, 2008, 2012 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc. 
#
from pykickstart.base import *
from pykickstart.errors import *
from pykickstart.options import *

from collections import OrderedDict

import gettext
_ = lambda x: gettext.ldgettext("pykickstart", x)

class FC3_AutoPart(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=100, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.autopart = kwargs.get("autopart", False)

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if self.autopart:
            retval += "autopart\n"

        return retval

    def parse(self, args):
        if len(args) > 0:
            raise KickstartValueError(formatErrorMsg(self.lineno, msg=_("Kickstart command %s does not take any arguments") % "autopart"))

        self.autopart = True
        return self

class F9_AutoPart(FC3_AutoPart):
    removedKeywords = FC3_AutoPart.removedKeywords
    removedAttrs = FC3_AutoPart.removedAttrs

    def __init__(self, writePriority=100, *args, **kwargs):
        FC3_AutoPart.__init__(self, writePriority=writePriority, *args, **kwargs)
        self.encrypted = kwargs.get("encrypted", False)
        self.passphrase = kwargs.get("passphrase", "")

        self.op = self._getParser()

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if not self.autopart:
            return retval

        retval += "autopart"

        if self.encrypted:
            retval += " --encrypted"

            if self.passphrase != "":
                retval += " --passphrase=\"%s\""% self.passphrase

        retval += "\n"
        return retval

    def _getParser(self):
        op = KSOptionParser()
        op.add_option("--encrypted", action="store_true", default=False)
        op.add_option("--passphrase")
        return op

    def parse(self, args):
        (opts, extra) = self.op.parse_args(args=args, lineno=self.lineno)
        # Rely on any error handling from baseclass
        FC3_AutoPart.parse(self, extra)

        self._setToSelf(self.op, opts)
        return self

class F12_AutoPart(F9_AutoPart):
    removedKeywords = F9_AutoPart.removedKeywords
    removedAttrs = F9_AutoPart.removedAttrs

    def __init__(self, writePriority=100, *args, **kwargs):
        F9_AutoPart.__init__(self, writePriority=writePriority, *args, **kwargs)

        self.escrowcert = kwargs.get("escrowcert", "")
        self.backuppassphrase = kwargs.get("backuppassphrase", False)

    def __str__(self):
        retval = F9_AutoPart.__str__(self)

        if not self.autopart:
            return retval

        if self.encrypted and self.escrowcert != "":
            retval = retval.strip()

            retval += " --escrowcert=\"%s\"" % self.escrowcert

            if self.backuppassphrase:
                retval += " --backuppassphrase"

            retval += "\n"

        return retval

    def _getParser(self):
        op = F9_AutoPart._getParser(self)
        op.add_option("--escrowcert")
        op.add_option("--backuppassphrase", action="store_true", default=False)
        return op

class RHEL6_AutoPart(F12_AutoPart):
    removedKeywords = F12_AutoPart.removedKeywords
    removedAttrs = F12_AutoPart.removedAttrs

    def __init__(self, writePriority=100, *args, **kwargs):
        F12_AutoPart.__init__(self, writePriority=writePriority, *args, **kwargs)
        self.cipher = kwargs.get("cipher", "")

    def __str__(self):
        retval = F12_AutoPart.__str__(self)
        if not self.autopart:
            return retval

        if self.encrypted and self.cipher:
            # remove any trailing newline
            retval = retval.strip()
            retval += " --cipher=\"%s\"" % self.cipher
            retval += "\n"

        return retval

    def _getParser(self):
        op = F12_AutoPart._getParser(self)
        op.add_option("--cipher")
        return op

    def parse(self, args):
        # call the overriden command to do it's job first
        retval = F12_AutoPart.parse(self, args)

        # Using autopart together with other partitioning command such as
        # part/partition, raid, logvol or volgroup can lead to hard to debug
        # behavior that might among other result into an unbootable system.
        #
        # Therefore if any of those commands is detected in the same kickstart
        # together with autopart, an error is raised and installation is
        # aborted.
        conflicting_command = ""

        # seen indicates that the corresponding
        # command has been seen in kickstart
        if self.handler.partition.seen:
            conflicting_command = "part/partition"
        elif self.handler.raid.seen:
            conflicting_command = "raid"
        elif self.handler.volgroup.seen:
            conflicting_command = "volgroup"
        elif self.handler.logvol.seen:
            conflicting_command = "logvol"

        if conflicting_command:
            # allow for translation of the error message
            errorMsg = _("The %s and autopart commands can't be used at the same time") % \
                         conflicting_command
            raise KickstartParseError(formatErrorMsg(self.lineno, msg=errorMsg))
        return retval


class F16_AutoPart(F12_AutoPart):
    removedKeywords = F12_AutoPart.removedKeywords
    removedAttrs = F12_AutoPart.removedAttrs

    def __init__(self, writePriority=100, *args, **kwargs):
        F12_AutoPart.__init__(self, writePriority=writePriority, *args, **kwargs)
        self.lvm = kwargs.get("lvm", True)

    def __str__(self):
        retval = F12_AutoPart.__str__(self)
        if not self.autopart:
            return retval

        # If requested, disable LVM autopart
        if not self.lvm:
            # remove any trailing newline
            retval = retval.strip()
            retval += " --nolvm"
            retval += "\n"

        return retval

    def _getParser(self):
        op = F12_AutoPart._getParser(self)
        op.add_option("--nolvm", action="store_false", dest="lvm",
            default=True)
        return op

class F17_AutoPart(F16_AutoPart):
    def __init__(self, writePriority=100, *args, **kwargs):
        F16_AutoPart.__init__(self, writePriority=writePriority, *args, **kwargs)
        self.type = kwargs.get("type", None)
        # This uses OrderedDict because we want to always output --type=plain
        # (as opposed to --type=partition) in __str__ when self.type is
        # AUTOPART_TYPE_PLAIN.
        self.typeMap = OrderedDict([("lvm", AUTOPART_TYPE_LVM),
                                    ("btrfs", AUTOPART_TYPE_BTRFS),
                                    ("plain", AUTOPART_TYPE_PLAIN),
                                    ("partition", AUTOPART_TYPE_PLAIN)])

    def __str__(self):
        retval = F16_AutoPart.__str__(self)
        if not self.autopart:
            return retval

        if self.type is not None:
            # remove any trailing newline
            retval = retval.strip()
            retval += " --type="
            for s, n in self.typeMap.items():
                if self.type == n:
                    retval += s
                    break
            retval += "\n"

        return retval

    def _getParser(self):
        def type_cb(option, opt_str, value, parser):
            if self.typeMap.has_key(value.lower()):
                parser.values.ensure_value(option.dest,
                                           self.typeMap[value.lower()])

        def nolvm_cb(option, opt_str, value, parser):
            parser.values.ensure_value(option.dest, AUTOPART_TYPE_PLAIN)

        op = F16_AutoPart._getParser(self)
        op.add_option("--nolvm", action="callback", callback=nolvm_cb,
                      dest="type", nargs=0)

        op.add_option("--type", action="callback", callback=type_cb,
                      dest="type", nargs=1, type="string")
        return op

    def parse(self, args):
        (opts, extra) = self.op.parse_args(args=args, lineno=self.lineno)
        # Rely on any error handling from baseclass
        F16_AutoPart.parse(self, extra)

        self._setToSelf(self.op, opts)

        # make this always True to avoid writing --nolvm
        self.lvm = True

        return self

class F18_AutoPart(F17_AutoPart):
    removedKeywords = F17_AutoPart.removedKeywords
    removedAttrs = F17_AutoPart.removedAttrs

    def __init__(self, writePriority=100, *args, **kwargs):
        F17_AutoPart.__init__(self, writePriority=writePriority, *args, **kwargs)
        self.cipher = kwargs.get("cipher", "")

    def __str__(self):
        retval = F17_AutoPart.__str__(self)
        if not self.autopart:
            return retval

        if self.encrypted and self.cipher:
            # remove any trailing newline
            retval = retval.strip()
            retval += " --cipher=\"%s\"" % self.cipher
            retval += "\n"

        return retval

    def _getParser(self):
        op = F17_AutoPart._getParser(self)
        op.add_option("--cipher")
        return op


class F20_AutoPart(F18_AutoPart):
    def __init__(self, writePriority=100, *args, **kwargs):
        F18_AutoPart.__init__(self, writePriority=writePriority, *args, **kwargs)
        self.typeMap["thinp"] = AUTOPART_TYPE_LVM_THINP

    def parse(self, args):
        # call the overriden command to do it's job first
        retval = F18_AutoPart.parse(self, args)

        # Using autopart together with other partitioning command such as
        # part/partition, raid, logvol or volgroup can lead to hard to debug
        # behavior that might among other result into an unbootable system.
        #
        # Therefore if any of those commands is detected in the same kickstart
        # together with autopart, an error is raised and installation is
        # aborted.
        conflicting_command = ""

        # seen indicates that the corresponding
        # command has been seen in kickstart
        if self.handler.partition.seen:
            conflicting_command = "part/partition"
        elif self.handler.raid.seen:
            conflicting_command = "raid"
        elif self.handler.volgroup.seen:
            conflicting_command = "volgroup"
        elif self.handler.logvol.seen:
            conflicting_command = "logvol"

        if conflicting_command:
            # allow for translation of the error message
            errorMsg = _("The %s and autopart commands can't be used at the same time") % \
                         conflicting_command
            raise KickstartParseError(formatErrorMsg(self.lineno, msg=errorMsg))
        return retval



import unittest
from tests.baseclass import *

from pykickstart.errors import *
from pykickstart.commands.selinux import *

class FC3_TestCase(CommandTest):
    command = "selinux"

    def runTest(self):
        # pass
        self.assert_parse("selinux")
        self.assert_parse("selinux --permissive", "selinux --permissive\n")
        self.assert_parse("selinux --enforcing", "selinux --enforcing\n")
        self.assert_parse("selinux --disabled", "selinux --disabled\n")

        # fail
        self.assert_parse_error("selinux --cheese")
        self.assert_parse_error("selinux --crackers=CRUNCHY")

if __name__ == "__main__":
    unittest.main()



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2007, 2009, 2013 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.base import *
from pykickstart.errors import *
from pykickstart.options import *

import gettext
_ = lambda x: gettext.ldgettext("pykickstart", x)

class F19_Liveimg(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, *args, **kwargs):
        KickstartCommand.__init__(self, *args, **kwargs)
        self.checksum = kwargs.get("checksum", "")
        self.noverifyssl = kwargs.get("noverifyssl", None)
        self.proxy = kwargs.get("proxy", None)
        self.url = kwargs.get("url", None)

        self.op = self._getParser()

    def __eq__(self, other):
        return self.url == other.url and self.proxy == other.proxy and \
               self.noverifyssl == other.noverifyssl and \
               self.checksum == other.checksum

    def __str__(self):
        retval = KickstartCommand.__str__(self)
        if not self.seen:
            return retval

        retval += "# Use live disk image installation\n"

        retval += "liveimg --url=\"%s\"" % self.url

        if self.proxy:
            retval += " --proxy=\"%s\"" % self.proxy

        if self.noverifyssl:
            retval += " --noverifyssl"

        if self.checksum:
            retval += " --checksum=\"%s\"" % self.checksum

        return retval + "\n"

    def _getParser(self):
        op = KSOptionParser()
        op.add_option("--url", required=1)
        op.add_option("--proxy")
        op.add_option("--noverifyssl", action="store_true", default=False)
        op.add_option("--checksum")
        return op

    def parse(self, args):
        (opts, extra) = self.op.parse_args(args=args, lineno=self.lineno)
        self._setToSelf(self.op, opts)

        return self



import unittest
import tempfile
from tests.baseclass import *

from pykickstart.version import *
from pykickstart.errors import *
from pykickstart.commands.vnc import *

def getClassName(cls):
    return cls().__class__.__name__

class StringToVersion_TestCase(CommandTest):
    def runTest(self):

        # fail - no version specified
        self.assertRaises(KickstartVersionError, stringToVersion, "RHEL")
        self.assertRaises(KickstartVersionError, stringToVersion, "Red Hat Enterprise Linux")
        self.assertRaises(KickstartVersionError, stringToVersion, "Fedora")
        self.assertRaises(KickstartVersionError, stringToVersion, "F")
        self.assertRaises(KickstartVersionError, stringToVersion, "FC")

        # fail - too old
        self.assertRaises(KickstartVersionError, stringToVersion, "Fedora Core 1")
        self.assertRaises(KickstartVersionError, stringToVersion, "Fedora Core 2")

        # fail - incorrect syntax
        self.assertRaises(KickstartVersionError, stringToVersion, "FC7")
        self.assertRaises(KickstartVersionError, stringToVersion, "FC8")
        self.assertRaises(KickstartVersionError, stringToVersion, "FC9")
        self.assertRaises(KickstartVersionError, stringToVersion, "FC10")
        self.assertRaises(KickstartVersionError, stringToVersion, "FC11")
        self.assertRaises(KickstartVersionError, stringToVersion, "F1111")
        self.assertRaises(KickstartVersionError, stringToVersion, "F 11")

        # pass - FC3
        self.assertEqual(stringToVersion("Fedora Core 3"), FC3)
        self.assertEqual(stringToVersion("FC3"), FC3)
        # pass - FC4
        self.assertEqual(stringToVersion("Fedora Core 4"), FC4)
        self.assertEqual(stringToVersion("FC4"), FC4)
        # pass - FC5
        self.assertEqual(stringToVersion("Fedora Core 5"), FC5)
        self.assertEqual(stringToVersion("FC5"), FC5)
        # pass - FC6
        self.assertEqual(stringToVersion("Fedora Core 6"), FC6)
        self.assertEqual(stringToVersion("FC6"), FC6)
        # pass - F7
        self.assertEqual(stringToVersion("Fedora Core 7"), F7)
        self.assertEqual(stringToVersion("Fedora 7"), F7)
        self.assertEqual(stringToVersion("F7"), F7)
        # pass - F8
        self.assertEqual(stringToVersion("Fedora Core 8"), F8)
        self.assertEqual(stringToVersion("Fedora 8"), F8)
        self.assertEqual(stringToVersion("F8"), F8)
        # pass - F9
        self.assertEqual(stringToVersion("Fedora Core 9"), F9)
        self.assertEqual(stringToVersion("Fedora 9"), F9)
        self.assertEqual(stringToVersion("F9"), F9)
        # pass - F10
        self.assertEqual(stringToVersion("Fedora Core 10"), F10)
        self.assertEqual(stringToVersion("Fedora 10"), F10)
        self.assertEqual(stringToVersion("F10"), F10)
        # pass - F11
        self.assertEqual(stringToVersion("Fedora 11"), F11)
        self.assertEqual(stringToVersion("F11"), F11)
        # pass - F12
        self.assertEqual(stringToVersion("Fedora 12"), F12)
        self.assertEqual(stringToVersion("F12"), F12)
        # pass - F13
        self.assertEqual(stringToVersion("Fedora 13"), F13)
        self.assertEqual(stringToVersion("F13"), F13)
        # pass - F14
        self.assertEqual(stringToVersion("Fedora 14"), F14)
        self.assertEqual(stringToVersion("F14"), F14)
        # pass - F15
        self.assertEqual(stringToVersion("Fedora 15"), F15)
        self.assertEqual(stringToVersion("F15"), F15)
        # pass - F16
        self.assertEqual(stringToVersion("Fedora 16"), F16)
        self.assertEqual(stringToVersion("F16"), F16)
        # pass - F17
        self.assertEqual(stringToVersion("Fedora 17"), F17)
        self.assertEqual(stringToVersion("F17"), F17)
        # pass - F18
        self.assertEqual(stringToVersion("Fedora 18"), F18)
        self.assertEqual(stringToVersion("F18"), F18)
        # pass - F19
        self.assertEqual(stringToVersion("Fedora 19"), F19)
        self.assertEqual(stringToVersion("F19"), F19)
        # pass - F20
        self.assertEqual(stringToVersion("Fedora 20"), F20)
        self.assertEqual(stringToVersion("F20"), F20)

        # pass - RHEL3
        self.assertEqual(stringToVersion("Red Hat Enterprise Linux 3"), RHEL3)
        self.assertEqual(stringToVersion("Red Hat Enterprise Linux AS 3"), RHEL3)
        self.assertEqual(stringToVersion("Red Hat Enterprise Linux ES 3"), RHEL3)
        self.assertEqual(stringToVersion("Red Hat Enterprise Linux WS 3"), RHEL3)
        self.assertEqual(stringToVersion("Red Hat Enterprise Linux Desktop 3"), RHEL3)
        self.assertEqual(stringToVersion("RHEL3"), RHEL3)

        # pass - RHEL4
        self.assertEqual(stringToVersion("Red Hat Enterprise Linux 4"), RHEL4)
        self.assertEqual(stringToVersion("Red Hat Enterprise Linux AS 4"), RHEL4)
        self.assertEqual(stringToVersion("Red Hat Enterprise Linux ES 4"), RHEL4)
        self.assertEqual(stringToVersion("Red Hat Enterprise Linux WS 4"), RHEL4)
        self.assertEqual(stringToVersion("Red Hat Enterprise Linux Desktop 4"), RHEL4)
        self.assertEqual(stringToVersion("RHEL4"), RHEL4)

        # pass - RHEL5
        self.assertEqual(stringToVersion("Red Hat Enterprise Linux 5"), RHEL5)
        self.assertEqual(stringToVersion("Red Hat Enterprise Linux Client 5"), RHEL5)
        self.assertEqual(stringToVersion("Red Hat Enterprise Linux Server 5"), RHEL5)
        for MINOR in range(1,10):
            self.assertEqual(stringToVersion("Red Hat Enterprise Linux 5.%s" % MINOR), RHEL5)
            self.assertEqual(stringToVersion("Red Hat Enterprise Linux Client 5.%s" % MINOR), RHEL5)
            self.assertEqual(stringToVersion("Red Hat Enterprise Linux Server 5.%s" % MINOR), RHEL5)
        self.assertEqual(stringToVersion("RHEL5"), RHEL5)

        # pass - RHEL6
        self.assertEqual(stringToVersion("Red Hat Enterprise Linux 6"), RHEL6)
        self.assertEqual(stringToVersion("Red Hat Enterprise Linux Client 6"), RHEL6)
        self.assertEqual(stringToVersion("Red Hat Enterprise Linux Server 6"), RHEL6)
        for MINOR in range(1,10):
            self.assertEqual(stringToVersion("Red Hat Enterprise Linux 6.%s" % MINOR), RHEL6)
            self.assertEqual(stringToVersion("Red Hat Enterprise Linux Client 6.%s" % MINOR), RHEL6)
            self.assertEqual(stringToVersion("Red Hat Enterprise Linux Server 6.%s" % MINOR), RHEL6)
        self.assertEqual(stringToVersion("RHEL6"), RHEL6)

        # pass - RHEL7
        self.assertEqual(stringToVersion("Red Hat Enterprise Linux 7"), RHEL7)
        self.assertEqual(stringToVersion("Red Hat Enterprise Linux Client 7"), RHEL7)
        self.assertEqual(stringToVersion("Red Hat Enterprise Linux Server 7"), RHEL7)
        for MINOR in range(1,10):
            self.assertEqual(stringToVersion("Red Hat Enterprise Linux 7.%s" % MINOR), RHEL7)
            self.assertEqual(stringToVersion("Red Hat Enterprise Linux Client 7.%s" % MINOR), RHEL7)
            self.assertEqual(stringToVersion("Red Hat Enterprise Linux Server 7.%s" % MINOR), RHEL7)
        self.assertEqual(stringToVersion("RHEL7"), RHEL7)

class VersionToString_TestCase(CommandTest):
    def runTest(self):

        # Make sure DEVEL is the highest version, but RHEL versions aren't
        # counted as devel.
        highest = 0
        for (ver_str,ver_num) in versionMap.items():
            if ver_str.startswith("RHEL"):
                continue

            highest = max(ver_num, highest)
        self.assertEqual(highest, DEVEL)

        # FC series
        self.assertEqual(versionToString(FC3), "FC3")
        self.assertEqual(versionToString(FC4), "FC4")
        self.assertEqual(versionToString(FC5), "FC5")
        self.assertEqual(versionToString(FC6), "FC6")
        # F series
        self.assertEqual(versionToString(F7), "F7")
        self.assertEqual(versionToString(F8), "F8")
        self.assertEqual(versionToString(F9), "F9")
        self.assertEqual(versionToString(F10), "F10")
        self.assertEqual(versionToString(F10, skipDevel=True), "F10")
        self.assertEqual(versionToString(F10, skipDevel=False), "F10")
        self.assertEqual(versionToString(F11, skipDevel=True), "F11")
        self.assertEqual(versionToString(F11, skipDevel=False), "F11")
        self.assertEqual(versionToString(F12, skipDevel=True), "F12")
        self.assertEqual(versionToString(F13, skipDevel=True), "F13")
        self.assertEqual(versionToString(F14, skipDevel=True), "F14")
        self.assertEqual(versionToString(F15, skipDevel=True), "F15")
        self.assertEqual(versionToString(F16, skipDevel=True), "F16")
        self.assertEqual(versionToString(F17, skipDevel=True), "F17")
        self.assertEqual(versionToString(F18, skipDevel=True), "F18")
        self.assertEqual(versionToString(F19, skipDevel=True), "F19")
        self.assertEqual(versionToString(F20, skipDevel=True), "F20")
        self.assertEqual(versionToString(F20, skipDevel=False), "DEVEL")
        # RHEL series
        self.assertEqual(versionToString(RHEL3), "RHEL3")
        self.assertEqual(versionToString(RHEL4), "RHEL4")
        self.assertEqual(versionToString(RHEL5), "RHEL5")
        self.assertEqual(versionToString(RHEL6), "RHEL6")
        self.assertEqual(versionToString(RHEL7), "RHEL7")

class returnClassForVersion_TestCase(CommandTest):
    def runTest(self):

        # Test that everything in version.versionMap has a handler, except
        # for DEVEL.
        for (name, vers) in versionMap.iteritems():
            if name == "DEVEL":
                continue

            self.assertEqual(returnClassForVersion(vers).version, vers)

        # Load the handlers
        import pykickstart.handlers
        for module in loadModules(pykickstart.handlers.__path__[0], cls_pattern="Handler", skip_list=["control"]):
            if module.__name__.endswith("Handler") and module.__name__ not in ["BaseHandler"]:
                # What is the version of the handler?
                vers = module.__name__.replace("Handler","")
                self.assertTrue(versionMap.has_key(vers))
                # Ensure that returnClassForVersion returns what we expect
                self.assertEqual(getClassName(returnClassForVersion(versionMap[vers])), getClassName(module))

class versionFromFile_TestCase(CommandTest):
    def runTest(self):

        def write_ks_cfg(buf):
            (fd, name) = tempfile.mkstemp(prefix="ks-", suffix=".cfg", dir="/tmp")
            os.write(fd, buf)
            os.close(fd)
            return name

        # no version specified
        ks_cfg = '''
# This is a sample kickstart file
rootpw testing123
cdrom
'''
        ks_cfg = write_ks_cfg(ks_cfg)
        self.assertEqual(versionFromFile(ks_cfg), DEVEL)
        os.unlink(ks_cfg)

        # proper format ... DEVEL
        ks_cfg = '''
# This is a sample kickstart file
#version=DEVEL
rootpw testing123
cdrom
'''
        ks_cfg = write_ks_cfg(ks_cfg)
        self.assertEqual(versionFromFile(ks_cfg), DEVEL)
        os.unlink(ks_cfg)

        # proper format ... RHEL3
        ks_cfg = '''
# This is a sample kickstart file
#version=RHEL3
rootpw testing123
cdrom
'''
        ks_cfg = write_ks_cfg(ks_cfg)
        self.assertEqual(versionFromFile(ks_cfg), RHEL3)
        os.unlink(ks_cfg)

        # improper format ... fallback to DEVEL
        ks_cfg = '''
# This is a sample kickstart file
# version: FC3
rootpw testing123
cdrom
'''
        ks_cfg = write_ks_cfg(ks_cfg)
        self.assertEqual(versionFromFile(ks_cfg), DEVEL)
        os.unlink(ks_cfg)

        # unknown version specified ... raise exception
        ks_cfg = '''
# This is a sample kickstart file
#version=RHEL5000
rootpw testing123
cdrom
'''
        ks_cfg = write_ks_cfg(ks_cfg)
        self.assertRaises(KickstartVersionError, versionFromFile, ks_cfg)
        os.unlink(ks_cfg)

if __name__ == "__main__":
    unittest.main()



import unittest, shlex
from tests.baseclass import *

from pykickstart.base import *
from pykickstart.errors import *
from pykickstart.version import *
from pykickstart.commands.volgroup import *

class FC3_TestCase(CommandTest):
    command = "volgroup"

    def runTest(self):
        # --noformat
        self.assert_parse("volgroup vg.01 --noformat",
                          "volgroup vg.01 --noformat --pesize=32768 --useexisting\n")
        # --useexisting
        self.assert_parse("volgroup vg.01 --useexisting",
                          "volgroup vg.01 --pesize=32768 --useexisting\n")

        # --pesize
        self.assert_parse("volgroup vg.01 pv.01 --pesize=70000",
                          "volgroup vg.01 --pesize=70000 pv.01\n")

        # assert data types
        self.assert_type("volgroup", "pesize", "int")
        self.assert_type("volgroup", "format", "boolean")
        self.assert_type("volgroup", "preexist", "boolean")

        # fail - incorrect type
        self.assert_parse_error("volgroup vg.01 pv.01 --pesize=SIZE", KickstartParseError)

        # fail - missing name
        self.assert_parse_error("volgroup", KickstartParseError)

        # fail - missing list of partitions
        self.assert_parse_error("volgroup vg01", KickstartValueError)

        # fail - both members and useexisting specified
        self.assert_parse_error("volgroup vg.01 --useexisting pv.01 pv.02", KickstartValueError)

class F16_TestCase(FC3_TestCase):
    def runTest(self):
        FC3_TestCase.runTest(self)

        # Pass - correct usage.
        self.assert_parse("volgroup vg.01 pv.01 --reserved-space=1000",
                          "volgroup vg.01 --pesize=32768 --reserved-space=1000 pv.01\n")
        self.assert_parse("volgroup vg.01 pv.01 --reserved-percent=50",
                          "volgroup vg.01 --pesize=32768 --reserved-percent=50 pv.01\n")

        # Fail - missing required argument.
        self.assert_parse_error("volgroup vg.01 pv.01 --reserved-space", KickstartParseError)
        self.assert_parse_error("volgroup vg.01 pv.01 --reserved-percent", KickstartParseError)

        # Fail - incorrect values.
        self.assert_parse_error("volgroup vg.01 pv.01 --reserved-space=-1", KickstartValueError)
        self.assert_parse_error("volgroup vg.01 pv.01 --reserved-percent=0", KickstartValueError)
        self.assert_parse_error("volgroup vg.01 pv.01 --reserved-percent=100", KickstartValueError)

if __name__ == "__main__":
    unittest.main()



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2005, 2006, 2007, 2012 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc. 
#
from pykickstart.base import *
from pykickstart.constants import *
from pykickstart.errors import *
from pykickstart.options import *

class FC3_ClearPart(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=120, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

        self.drives = kwargs.get("drives", [])
        self.initAll = kwargs.get("initAll", False)
        self.type = kwargs.get("type", None)

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if self.type is None:
            return retval

        if self.type == CLEARPART_TYPE_NONE:
            clearstr = "--none"
        elif self.type == CLEARPART_TYPE_LINUX:
            clearstr = "--linux"
        elif self.type == CLEARPART_TYPE_ALL:
            clearstr = "--all"
        else:
            clearstr = ""

        if self.initAll:
            initstr = "--initlabel"
        else:
            initstr = ""

        if len(self.drives) > 0:
            drivestr = "--drives=" + ",".join(self.drives)
        else:
            drivestr = ""

        retval += "# Partition clearing information\nclearpart %s %s %s\n" % (clearstr, initstr, drivestr)
        return retval

    def _getParser(self):
        def drive_cb (option, opt_str, value, parser):
            for d in value.split(','):
                parser.values.ensure_value(option.dest, []).append(d)

        op = KSOptionParser()
        op.add_option("--all", dest="type", action="store_const",
                      const=CLEARPART_TYPE_ALL)
        op.add_option("--drives", dest="drives", action="callback",
                      callback=drive_cb, nargs=1, type="string")
        op.add_option("--initlabel", dest="initAll", action="store_true",
                      default=False)
        op.add_option("--linux", dest="type", action="store_const",
                      const=CLEARPART_TYPE_LINUX)
        op.add_option("--none", dest="type", action="store_const",
                      const=CLEARPART_TYPE_NONE)
        return op

    def parse(self, args):
        (opts, extra) = self.op.parse_args(args=args, lineno=self.lineno)
        self._setToSelf(self.op, opts)
        return self

class F17_ClearPart(FC3_ClearPart):
    def __init__(self, *args, **kwargs):
        super(F17_ClearPart, self).__init__(*args, **kwargs)
        self.devices = kwargs.get("devices", [])

    def __str__(self):
        s = super(F17_ClearPart, self).__str__()
        if s and len(self.devices) > 0:
            s = s.rstrip()
            s += " --list=" + ",".join(self.devices)
            s += "\n"
        return s

    def _getParser(self):
        op = FC3_ClearPart._getParser(self)

        def list_cb (option, opt_str, value, parser):
            self.type = CLEARPART_TYPE_LIST
            for d in value.split(','):
                parser.values.ensure_value(option.dest, []).append(d)

        op.add_option("--list", dest="devices", action="callback",
                      callback=list_cb, nargs=1, type="string")
        return op



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2005, 2006, 2007 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc. 
#
from pykickstart.base import *
from pykickstart.errors import *
from pykickstart.options import *

import gettext
_ = lambda x: gettext.ldgettext("pykickstart", x)

class FC3_RootPw(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

        self.isCrypted = kwargs.get("isCrypted", False)
        self.password = kwargs.get("password", "")

    def _getArgsAsStr(self):
        retval = ""

        if self.isCrypted:
            retval += " --iscrypted"

        return retval

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if self.password != "":
            retval += "# Root password\nrootpw%s %s\n" % (self._getArgsAsStr(), self.password)

        return retval

    def _getParser(self):
        op = KSOptionParser()
        op.add_option("--iscrypted", dest="isCrypted", action="store_true",
                      default=False)
        return op

    def parse(self, args):
        (opts, extra) = self.op.parse_args(args=args, lineno=self.lineno)
        self._setToSelf(self.op, opts)

        if len(extra) != 1:
            raise KickstartValueError(formatErrorMsg(self.lineno, msg=_("A single argument is expected for the %s command") % "rootpw"))

        self.password = extra[0]
        return self

class F8_RootPw(FC3_RootPw):
    removedKeywords = FC3_RootPw.removedKeywords
    removedAttrs = FC3_RootPw.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        FC3_RootPw.__init__(self, writePriority, *args, **kwargs)
        self.lock = kwargs.get("lock", False)

    def _getArgsAsStr(self):
        retval = FC3_RootPw._getArgsAsStr(self)

        if self.lock:
            retval += " --lock"

        if not self.isCrypted:
            retval += " --plaintext"

        return retval

    def _getParser(self):
        op = FC3_RootPw._getParser(self)
        op.add_option("--lock", dest="lock", action="store_true", default=False)
        op.add_option("--plaintext", dest="isCrypted", action="store_false")
        return op

class F18_RootPw(F8_RootPw):
    removedKeywords = F8_RootPw.removedKeywords
    removedAttrs = F8_RootPw.removedAttrs

    def __str__(self):
        retval = F8_RootPw.__str__(self)

        if not retval and self.lock:
            retval = "#Root password\nrootpw --lock\n"

        return retval

    def parse(self, args):
        (opts, extra) = self.op.parse_args(args=args, lineno=self.lineno)
        self._setToSelf(self.op, opts)

        if len(extra) != 1 and not self.lock:
            raise KickstartValueError(formatErrorMsg(self.lineno, msg=_("A single argument is expected for the %s command") % "rootpw"))

        if len(extra) == 1:
            self.password = extra[0]

        return self



#
# Chris Lumens <clumens@redhat.com>
# David Lehman <dlehman@redhat.com>
#
# Copyright 2005, 2006, 2007, 2011 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc. 
#
from pykickstart.base import *
from pykickstart.errors import *
from pykickstart.options import *

import gettext
import warnings
_ = lambda x: gettext.ldgettext("pykickstart", x)

class F17_BTRFSData(BaseData):
    removedKeywords = BaseData.removedKeywords
    removedAttrs = BaseData.removedAttrs

    def __init__(self, *args, **kwargs):
        BaseData.__init__(self, *args, **kwargs)
        self.format = kwargs.get("format", True)
        self.preexist = kwargs.get("preexist", False)
        self.label = kwargs.get("label", "")
        self.mountpoint = kwargs.get("mountpoint", "")
        self.devices = kwargs.get("devices", [])
        self.dataLevel = kwargs.get("data", None)
        self.metaDataLevel = kwargs.get("metadata", None)

        # subvolume-specific
        self.subvol = kwargs.get("subvol", False)
        self.name = kwargs.get("name", None)        # required

    def __eq__(self, y):
        if not y:
            return False

        return self.mountpoint == y.mountpoint

    def __ne__(self, y):
        return not self == y

    def _getArgsAsStr(self):
        retval = ""
        if not self.format:
            retval += " --noformat"
        if self.preexist:
            retval += " --useexisting"
        if self.label:
            retval += " --label=%s" % self.label
        if self.dataLevel:
            retval += " --data=%s" % self.dataLevel
        if self.metaDataLevel:
            retval += " --metadata=%s" % self.metaDataLevel
        if self.subvol:
            retval += " --subvol --name=%s" % self.name

        return retval

    def __str__(self):
        retval = BaseData.__str__(self)
        retval += "btrfs %s" % self.mountpoint
        retval += self._getArgsAsStr()
        return retval + " " + " ".join(self.devices) + "\n"

class F17_BTRFS(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=132, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

        # A dict of all the RAID levels we support.  This means that if we
        # support more levels in the future, subclasses don't have to
        # duplicate too much.
        self.levelMap = { "RAID0": "raid0", "0": "raid0",
                          "RAID1": "raid1", "1": "raid1",
                          "RAID10": "raid10", "10": "raid10",
                          "single": "single" }

        self.btrfsList = kwargs.get("btrfsList", [])

    def __str__(self):
        retval = ""
        for btr in self.btrfsList:
            retval += btr.__str__()

        return retval

    def _getParser(self):
        # Have to be a little more complicated to set two values.
        def btrfs_cb (option, opt_str, value, parser):
            parser.values.format = False
            parser.values.preexist = True

        def level_cb (option, opt_str, value, parser):
            if self.levelMap.has_key(value):
                parser.values.ensure_value(option.dest, self.levelMap[value])

        op = KSOptionParser()
        op.add_option("--noformat", action="callback", callback=btrfs_cb,
                      dest="format", default=True, nargs=0)
        op.add_option("--useexisting", action="callback", callback=btrfs_cb,
                      dest="preexist", default=False, nargs=0)

        # label, data, metadata
        op.add_option("--label", dest="label", default="")
        op.add_option("--data", dest="dataLevel", action="callback",
                      callback=level_cb, type="string", nargs=1)
        op.add_option("--metadata", dest="metaDataLevel", action="callback",
                      callback=level_cb, type="string", nargs=1)

        #
        # subvolumes
        #
        op.add_option("--subvol", dest="subvol", action="store_true",
                      default=False)

        # parent must be a device spec (LABEL, UUID, &c)
        op.add_option("--parent", dest="parent", default="")
        op.add_option("--name", dest="name", default="")

        return op

    def parse(self, args):
        (opts, extra) = self.op.parse_args(args=args, lineno=self.lineno)
        data = self.handler.BTRFSData()
        self._setToObj(self.op, opts, data)
        data.lineno = self.lineno

        if len(extra) == 0:
            raise KickstartValueError(formatErrorMsg(self.lineno, msg=_("btrfs must be given a mountpoint")))

        if len(extra) == 1 and not data.subvol:
            raise KickstartValueError(formatErrorMsg(self.lineno, msg=_("btrfs must be given a list of partitions")))
        elif len(extra) == 1:
            raise KickstartValueError(formatErrorMsg(self.lineno, msg=_("btrfs subvol requires specification of parent volume")))

        if data.subvol and not data.name:
            raise KickstartValueError(formatErrorMsg(self.lineno, msg=_("btrfs subvolume requires a name")))

        data.mountpoint = extra[0]
        data.devices = extra[1:]

        # Check for duplicates in the data list.
        if data in self.dataList():
            warnings.warn(_("A btrfs volume with the mountpoint %s has already been defined.") % data.label)

        return data

    def dataList(self):
        return self.btrfsList



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2005, 2006, 2007 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc. 
#
from pykickstart.base import *
from pykickstart.errors import *
from pykickstart.options import *

import gettext
_ = lambda x: gettext.ldgettext("pykickstart", x)

class FC3_Upgrade(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.upgrade = kwargs.get("upgrade", None)
        self.op = self._getParser()

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if self.upgrade is None:
            return retval

        if self.upgrade:
            retval += "# Upgrade existing installation\nupgrade\n"
        else:
            retval += "# Install OS instead of upgrade\ninstall\n"

        return retval

    def _getParser(self):
        op = KSOptionParser()
        return op

    def parse(self, args):
        (opts, extra) = self.op.parse_args(args=args, lineno=self.lineno)

        if len(extra) > 0:
            raise KickstartValueError(formatErrorMsg(self.lineno, msg=_("Kickstart command %s does not take any arguments") % "upgrade"))

        if self.currentCmd == "upgrade":
            self.upgrade = True
        else:
            self.upgrade = False

        return self

class F11_Upgrade(FC3_Upgrade):
    removedKeywords = FC3_Upgrade.removedKeywords
    removedAttrs = FC3_Upgrade.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        FC3_Upgrade.__init__(self, writePriority, *args, **kwargs)

        self.op = self._getParser()
        self.root_device = kwargs.get("root_device", None)

    def __str__(self):
        if self.upgrade and (self.root_device is not None):
            retval = KickstartCommand.__str__(self)
            retval += "# Upgrade existing installation\nupgrade --root-device=%s\n" % self.root_device
        else:
            retval = FC3_Upgrade.__str__(self)

        return retval

    def _getParser(self):
        op = KSOptionParser()
        op.add_option("--root-device", dest="root_device")
        return op

    def parse(self, args):
        (opts, extra) = self.op.parse_args(args=args, lineno=self.lineno)

        if len(extra) > 0:
            raise KickstartValueError(formatErrorMsg(self.lineno, msg=_("Kickstart command %s does not take any arguments") % "upgrade"))

        if (opts.root_device is not None) and (opts.root_device == ""):
            raise KickstartValueError(formatErrorMsg(self.lineno, msg=_("Kickstart command %s does not accept empty parameter %s") % ("upgrade", "--root-device")))
        else:
            self.root_device = opts.root_device

        if self.currentCmd == "upgrade":
            self.upgrade = True
        else:
            self.upgrade = False

        return self

class F20_Upgrade(DeprecatedCommand):
    def __init__(self):
        DeprecatedCommand.__init__(self)



#
# Martin Gracik <mgracik@redhat.com>
#
# Copyright 2009 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#

import unittest
from tests.baseclass import *

class FC3_TestCase(CommandTest):
    command = "method"

    def runTest(self):
        # pass
        # cdrom
        self.assert_parse("cdrom", "cdrom\n")

        # harddrive
        self.assert_parse("harddrive --dir=/install --biospart=part", "harddrive --dir=/install --biospart=part\n")
        self.assert_parse("harddrive --dir=/install --partition=part", "harddrive --dir=/install --partition=part\n")

        # nfs
        self.assert_parse("nfs --server=1.2.3.4 --dir=/install", "nfs --server=1.2.3.4 --dir=/install\n")

        # url
        self.assert_parse("url --url=http://domain.com", "url --url=\"http://domain.com\"\n")

        # fail
        # harddrive
        # required option --dir missing
        self.assert_parse_error("harddrive", KickstartValueError)
        # required --dir argument missing
        self.assert_parse_error("harddrive --dir", KickstartParseError)
        # missing --biospart or --partition option
        self.assert_parse_error("harddrive --dir=/install", KickstartValueError)
        # both --biospart and --partition specified
        self.assert_parse_error("harddrive --dir=/install --biospart=bios --partition=part", KickstartValueError)
        # --biospart and --partition require argument
        self.assert_parse_error("harddrive --dir=/install --biospart", KickstartParseError)
        self.assert_parse_error("harddrive --dir=/install --partition", KickstartParseError)
        # unknown option
        self.assert_parse_error("harddrive --unknown=value", KickstartParseError)

        # nfs
        # missing required options --server and --dir
        self.assert_parse_error("nfs", KickstartValueError)
        self.assert_parse_error("nfs --server=1.2.3.4", KickstartValueError)
        self.assert_parse_error("nfs --server", KickstartParseError)
        self.assert_parse_error("nfs --dir=/install", KickstartValueError)
        self.assert_parse_error("nfs --dir", KickstartParseError)
        # unknown option
        self.assert_parse_error("nfs --unknown=value", KickstartParseError)

        # url
        # missing required option --url
        self.assert_parse_error("url", KickstartValueError)
        self.assert_parse_error("url --url", KickstartParseError)


class FC6_TestCase(FC3_TestCase):
    def runTest(self):
        # run FC3 test case
        FC3_TestCase.runTest(self)

        # pass
        # nfs
        self.assert_parse("nfs --server=1.2.3.4 --dir=/install --opts=options", "nfs --server=1.2.3.4 --dir=/install --opts=\"options\"\n")

        # fail
        # nfs
        # --opts requires argument if specified
        self.assert_parse_error("nfs --server=1.2.3.4 --dir=/install --opts", KickstartParseError)


class F13_TestCase(FC6_TestCase):
    def runTest(self):
        # run FC6 test case
        FC6_TestCase.runTest(self)

        # pass
        self.assert_parse("url --url=http://someplace/somewhere --proxy=http://wherever/other",
                          "url --url=\"http://someplace/somewhere\" --proxy=\"http://wherever/other\"\n")

        # fail
        self.assert_parse_error("cdrom --proxy=http://someplace/somewhere", KickstartParseError)
        self.assert_parse_error("url --url=http://someplace/somewhere --proxy", KickstartParseError)
        self.assert_parse_error("url --proxy=http://someplace/somewhere", KickstartValueError)

class F14_TestCase(F13_TestCase):
    def runTest(self):
        # run FC6 test case
        F13_TestCase.runTest(self)

        # pass
        self.assert_parse("url --url=https://someplace/somewhere --noverifyssl",
                          "url --url=\"https://someplace/somewhere\" --noverifyssl\n")

        # fail
        self.assert_parse_error("cdrom --noverifyssl", KickstartParseError)

class F18_TestCase(F14_TestCase):
    def runTest(self):
        # run F14 test case.
        F14_TestCase.runTest(self)

        # pass
        self.assert_parse("url --mirrorlist=http://www.wherever.com/mirror",
                          "url --mirrorlist=\"http://www.wherever.com/mirror\"\n")

        # fail
        # missing one of required options --url or --mirrorlist
        self.assert_parse_error("url", KickstartValueError)
        self.assert_parse_error("url --mirrorlist", KickstartParseError)

        # It's --url, not --baseurl.
        self.assert_parse_error("url --baseurl=www.wherever.com", KickstartParseError)

        # only one of --url or --mirrorlist may be specified
        self.assert_parse_error("url --url=www.wherever.com --mirrorlist=www.wherever.com",
                                KickstartValueError)

class F19_TestCase(F18_TestCase):
    def runTest(self):
        # run F18 test case.
        F18_TestCase.runTest(self)

        # liveimg pass
        self.assert_parse("liveimg --url=http://someplace/somewhere --proxy=http://wherever/other "
                          "--noverifyssl --checksum=e7a9fe500330a1cae4ca114833bb3df014e6d14e63ea9566896a848f3832d0ba",
                          "liveimg --url=\"http://someplace/somewhere\" --proxy=\"http://wherever/other\" "
                          "--noverifyssl --checksum=\"e7a9fe500330a1cae4ca114833bb3df014e6d14e63ea9566896a848f3832d0ba\"\n")
        self.assert_parse("liveimg --url=http://someplace/somewhere --proxy=http://wherever/other "
                          "--noverifyssl",
                          "liveimg --url=\"http://someplace/somewhere\" --proxy=\"http://wherever/other\" "
                          "--noverifyssl\n")
        self.assert_parse("liveimg --url=http://someplace/somewhere --proxy=http://wherever/other ",
                          "liveimg --url=\"http://someplace/somewhere\" --proxy=\"http://wherever/other\"\n")
        self.assert_parse("liveimg --url=http://someplace/somewhere",
                          "liveimg --url=\"http://someplace/somewhere\"\n")

        # liveimg fail
        self.assert_parse_error("liveimg", KickstartValueError)
        self.assert_parse_error("liveimg --url", KickstartParseError)
        self.assert_parse_error("liveimg --url=http://someplace/somewhere --proxy", KickstartParseError)
        self.assert_parse_error("liveimg --proxy=http://someplace/somewhere", KickstartValueError)
        self.assert_parse_error("liveimg --noverifyssl", KickstartValueError)
        self.assert_parse_error("liveimg --checksum=e7a9fe500330a1cae4ca114833bb3df014e6d14e63ea9566896a848f3832d0ba", KickstartValueError)


if __name__ == "__main__":
    unittest.main()
